<?php

/**
 * CTax.php - Implementation file for Tax Module class.
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

namespace SDExtension;

class CTax
	{

	/**
	 * @var object $m_oCatalogDB Catalog DB
	 */
	protected $m_oCatalogDB = null;

	/**
	 * @var object $m_oShippingDB Shipping DB
	 */
	protected $m_oShippingDB = null;

	/**
	 * @var string $m_sCountry Country
	 */
	protected $m_sCountry = null;

	/**
	 * @var array $m_aTaxDefinition Tax Definition
	 */
	protected $m_aTaxDefinition = null;

	/**
	 * @var array $m_aTaxModel Tax Model
	 */
	protected $m_aTaxModel = null;

	/**
	 * @var array $m_aTaxBandNames Tax Band Names
	 */
	protected $m_aTaxBandNames = [];

	/**
	 * @var array $m_aProductTaxes Product Taxes
	 */
	protected $m_aProductTaxes = [];

	/**
	 * @var array $m_nCustomTaxNo Custom Tax No
	 */
	protected $m_nCustomTaxNo = 0;

	/**
	 * @var array $m_aRoundingRules Rounding Rules
	 */
	protected $m_aRoundingRules = [
		TAX_ROUND_TRUNCATION => "down",
		TAX_ROUND_SCIENTIFIC_DOWN => "halfdown",
		TAX_ROUND_SCIENTIFIC_NORMAL => "normal",
		TAX_ROUND_CEILING => "up",
		TAX_ROUND_BANKERS => "bankers",
	];

	/**
	 * @var array $m_aRoundingGroups Rounding Groups
	 */
	protected $m_aRoundingGroups = [
		TAX_ROUND_PER_LINE => "line",
		TAX_ROUND_PER_ITEM => "item",
		TAX_ROUND_PER_ORDER => "order",
	];

	/**
	 * @var bool $m_bSimpleTax TRUE if Simple Tax applies, FALSE, if Advanced
	 */
	protected $m_bSimpleTax = true;

	/**
	 * __construct - Object constructor method
	 *
	 * @access public
	 * @param object $oCatalogDB Catalog Database Object
	 * @param object $oShippingDB Shipping Database Object
	 * @return void
	 */
	public function __construct($oCatalogDB, $oShippingDB)
		{
		$this->m_oCatalogDB = $oCatalogDB;
		$this->m_oShippingDB = $oShippingDB;
		}

	/**
	 * GetCountry - Gets Country from DB
	 *
	 * @access public
	 * @return string Country
	 */
	public function GetCountry()
		{
		if ($this->m_sCountry == null)
			{
			$oSetupQueryResult = $this->m_oCatalogDB->Select('Setup', ['Country']);
			$this->m_sCountry = select_get($oSetupQueryResult, 'Country', 0);
			}
		return $this->m_sCountry;
		}

	/**
	 * GetTaxDefinition - Gets Tax Definition
	 *
	 * @access public
	 * @return array Tax Definition
	 */
	public function GetTaxDefinition()
		{
		return $this->m_aTaxDefinition;
		}

	/**
	 * GenerateOpaqueShipDataElements - Generates Opaque Shipping Data Elements
	 *
	 * @access public
	 * @param string $sCountryName Country name
	 * @return string Opaque Shipping Data
	 */
	public function GenerateOpaqueShipDataElements($sCountryName)
		{
		$oQueryResult = $this->m_oCatalogDB->Select('Countries', ['nCountryID'], [['sCountryCode', $sCountryName]]);
		$nCountryID = select_get($oQueryResult, 'nCountryID', 0);
		$oQueryResult = $this->m_oShippingDB->Select('ZoneMembers', ['ZoneID'], [['RegionID', $nCountryID]]);
		$nZoneID = select_get($oQueryResult, 'ZoneID');
		return [$nCountryID, $nZoneID];
		}

	/**
	 * GenerateOpaqueShipData - Generates Opaque Shipping Data
	 *
	 * @access public
	 * @param string $sCountry Country name
	 * @param int $nCost Shipping Cost
	 * @return string Opaque Shipping Data
	 */
	public function GenerateOpaqueShipData($sCountry, $nCost)
		{
		list($nCountryID, $nZoneID) = $this->GenerateOpaqueShipDataElements($sCountry);
		$sOpaqueShipData = "ShippingClass;Unknown;ShippingZone;$nZoneID;BasisTotal;0.000000;Simple;$nCost;NoRecalculation;";
		return $sOpaqueShipData;
		}

	/**
	 * GetInternationalShipping - Gets International Shipping
	 *
	 * Find the matching value in sCountryCode in the Countries table of the SellerDeck database and lookup nCountryID for the record
	 * Cross reference nCountryID against RegionID in the ZoneMembers table of the shipping database to get the zone name (ZoneID)
	 * For each Zone containing the country, look up by name (ZoneID) against the ZoneName in the Zones table.
	 * If ANY zone containing the country has bInternational set to -1, set the international shipping flag for the order (see SD-2782).
	 *
	 * @access public
	 * @param string $sCountryName Country name
	 * @return bool Opaque Shipping Data
	 */
	public function GetInternationalShipping($sCountryName)
		{
		list($nCountryID, $nZoneID) = $this->GenerateOpaqueShipDataElements($sCountryName);
		$oQueryResult = $this->m_oShippingDB->Select('Zones', '*', [['ZoneID', $nZoneID], ['bInternational', true]]);
		if (is_object($oQueryResult))
			{
			$aZones = $oQueryResult->fetch(\PDO::FETCH_ASSOC);
			return !empty($aZones);
			}
		return false;
		}

	/**
	 * GenerateOpaqueShipDataForOrderDetail - Generates Opaque Shipping Data for OrderDetail
	 *
	 * @access public
	 * @param array $aProduct Product array
	 * @return string Opaque Shipping Data
	 */
	public function GenerateOpaqueShipDataForOrderDetail($aProduct)
		{
		return ";ALT_WEIGHT=" . arr_get($aProduct, 'sAltWeight', '')
				. ";EXCLUDE_FROM_SHIP=" . abs(arr_get($aProduct, 'bExcludeFromShippingCalc', 0))
				. ";SHIP_CATEGORY=" . arr_get($aProduct, 'sShippingCategory', 0)
				. ";SHIP_QUANTITY=" . arr_get($aProduct, 'nShippingQuantity', 1)
				. ";SHIP_SUPPLEMENT=" . arr_get($aProduct, 'dShippingSupplement', 0)
				. ";SHIP_SUPPLEMENT_ONCE=" . arr_get($aProduct, 'bShipSupplAppliedOnce', 0)
				. ";HAND_SUPPLEMENT=" . arr_get($aProduct, 'dHandlingSupplement', 0)
				. ";HAND_SUPPLEMENT_ONCE=" . arr_get($aProduct, 'bHandSupplAppliedOnce', 0)
				. ";EXCLUDE_PARENT=" . arr_get($aProduct, '', 0)
				. ";SEP_LINE=" . arr_get($aProduct, 'bShipSeparately', '')
				. ";USE_ASSOC_SHIP=" . arr_get($aProduct, '', '')
				. ";";
		}

	/**
	 * GenerateTaxModelOpaqueData - Generates Tax Model Opaque Data
	 *
	 *   Tax  model information
	 *
	 *   eTaxModel=eTaxBy=Shipping Tax 1 Opaque data=Shipping Tax 2 Opaque Data=Handling Tax 1 Opaque Data=Handling Tax 2 Opaque Data
	 *
	 *   eTaxModel	   0 - simple
	 * 			   1 - advanced
	 *
	 *   eTaxBy	   0 - always tax
	 * 			   1 - tax by invoice address
	 * 			   2 - tax by delivery address
	 *
	 * 	example: 1=2=0=0=0=Zero-Rated=0=0=0=??=0=0=0=??=0=0=0=??=
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @return string Tax Model Opaque Data
	 */
	public function GenerateTaxModelOpaqueData($sCountryName)
		{
		if (null == $sCountryName)
			{
			return "0=0=0=0=0==0=0=0==0=0=0==0=0=0==";
			}
		if ($this->m_bSimpleTax)
			{
			$nTaxModel = TAX_MODEL_SIMPLE;
			$aTaxes = $this->GetSimpleTaxes($this->GetTaxModel());
			}
		else
			{
			$nTaxModel = TAX_MODEL_ADVANCED;
			$aTaxes = $this->GetCountryTaxes($sCountryName);
			}

		$nTaxBy = TAX_BY_ALWAYS_TAX;
		$sShippingTax1OpaqueData = arr_get($aTaxes, '1.sShippingOpaqueData', "0=0=0=");
		$sShippingTax2OpaqueData = arr_get($aTaxes, '2.sShippingOpaqueData', "0=0=0=");
		$sHandlingTax1OpaqueData = arr_get($aTaxes, '1.sHandlingOpaqueData', "0=0=0=");
		$sHandlingTax2OpaqueData = arr_get($aTaxes, '2.sHandlingOpaqueData', "0=0=0=");
		$sShippingTax1BandName = arr_get($this->m_aTaxBandNames, $this->ParseProductTaxData($sShippingTax1OpaqueData)->nBandID, '');
		$sShippingTax2BandName = arr_get($this->m_aTaxBandNames, $this->ParseProductTaxData($sShippingTax2OpaqueData)->nBandID, '');
		$sHandlingTax1BandName = arr_get($this->m_aTaxBandNames, $this->ParseProductTaxData($sHandlingTax1OpaqueData)->nBandID, '');
		$sHandlingTax2BandName = arr_get($this->m_aTaxBandNames, $this->ParseProductTaxData($sHandlingTax2OpaqueData)->nBandID, '');
		if (empty(trim($sShippingTax1OpaqueData)))
			{
			$sShippingTax1OpaqueData = "0=0=0=";
			}
		if (empty(trim($sShippingTax2OpaqueData)))
			{
			$sShippingTax2OpaqueData = "0=0=0=";
			}
		if (empty(trim($sHandlingTax1OpaqueData)))
			{
			$sHandlingTax1OpaqueData = "0=0=0=";
			}
		if (empty(trim($sHandlingTax2OpaqueData)))
			{
			$sHandlingTax2OpaqueData = "0=0=0=";
			}
		$sTaxModelOpaqueData = "$nTaxModel=$nTaxBy=$sShippingTax1OpaqueData$sShippingTax1BandName=$sShippingTax2OpaqueData$sShippingTax2BandName=$sHandlingTax1OpaqueData$sHandlingTax1BandName=$sHandlingTax2OpaqueData$sHandlingTax2BandName=";
		return $sTaxModelOpaqueData;
		}

	/**
	 * IsTaxing - Do we include tax, when simple tax is selected?
	 *
	 * @access public
	 * @return array Tax query result
	 */
	public function IsTaxing()
		{
		$oQueryResult = $this->m_oCatalogDB->Select(
				'TaxSetup', [['TaxZones', 'bIncludeTax1']], [[['TaxSetup', 'nModelID'], 0]], [
			'join' => [
				['left', 'TaxZones', [[['TaxZones', 'nModelID'], ['TaxSetup', 'nModelID']]]],
			],
				]
		);
		if ($oQueryResult === false)
			{
			return false;
			}
		$aTaxZonesResult = $oQueryResult->fetch(\PDO::FETCH_ASSOC);
		if ($oQueryResult === false)
			{
			//
			// Advanced Taxing
			//
			return true;
			}
		$bIncludeTax1 = (bool) abs(arr_get($aTaxZonesResult, 'bIncludeTax1', true));
		return $bIncludeTax1;
		}

	/**
	 * GetSimpleTaxes - Get Simple Taxes
	 *
	 * @access public
	 * @param array $aTaxModel Tax Model details
	 * @return array Tax query result
	 */
	public function GetSimpleTaxes($aTaxModel)
		{
		$aTaxes = [];
		for ($nIndex = 1; $nIndex <= 2; $nIndex++)
			{
			if (arr_get($aTaxModel, 'bIncludeTax' . $nIndex . 'Always', 1) != 0)
				{
				$oQueryResult = $this->m_oCatalogDB->Select(
						'Taxes', [['Taxes', '*']], [['nTaxID', arr_get($aTaxModel, 'nTax' . $nIndex . 'ID', 1)]]
				);
				if ($oQueryResult === false)
					{
					return null;
					}
				$aTaxes[$nIndex] = $oQueryResult->fetch(\PDO::FETCH_ASSOC);
				}
			}
		return $aTaxes;
		}

	/**
	 * GetCountryTaxes - Get Country Taxes
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @return array Tax query result
	 */
	public function GetCountryTaxes($sCountryName)
		{
		return array_filter([
			1 => $this->GetCountryTax($sCountryName, 1),
			2 => $this->GetCountryTax($sCountryName, 2),
		]);
		}

	/**
	 * GetTaxModel - Get Tax Model
	 *
	 * @access public
	 * @return array|bool Tax query result
	 */
	public function GetTaxModel()
		{
		if ($this->m_aTaxModel == null)
			{
			$oQueryResult = $this->m_oCatalogDB->Select(
					'TaxSetup', [['TaxModels', '*'], 'sShippingTax1OpaqueData', 'sShippingTax2OpaqueData', 'sHandlingTax1OpaqueData', 'sHandlingTax2OpaqueData', 'bTaxZonesRestrictive', 'bTaxInclusiveMode'], [], [
				'join' => [
					['left', 'TaxModels', [[['TaxSetup', 'nModelID'], ['TaxModels', 'nModelID']]]],
				],
					]
			);
			if ($oQueryResult === false)
				{
				return null;
				}
			return $oQueryResult->fetch(\PDO::FETCH_ASSOC);
			}
		else
			{
			return $this->m_aTaxModel;
			}
		}

	/**
	 * GetCountryTax - Get Country Tax
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @return array|bool Tax query result
	 */
	public function GetCountryTax($sCountryName, $nTaxNo = 1)
		{
		if (!in_array($nTaxNo, [1, 2]))
			{
			return null;
			}
		$oQueryResult = $this->m_oCatalogDB->Select(
				'Taxes', [['Taxes', '*'], ['TaxZones', 'bAllowTax1UserExemption'], ['TaxZones', 'bAllowTax2UserExemption'], ['TaxZones', 'bTaxOtherTaxes']], [['sCountryName', $sCountryName]], [
			'join' => [
				['left', 'TaxZones', [[['Taxes', 'nTaxID'], ['TaxZones', 'nTax' . $nTaxNo . 'ID']]]],
				['left', 'TaxZoneMembers', [[['TaxZones', 'nZoneID'], ['TaxZoneMembers', 'nTaxZoneID']]]],
				['left', 'Countries', [[['TaxZoneMembers', 'nRegionID'], ['Countries', 'nCountryID']]]],
			],
				]
		);
		if ($oQueryResult === false)
			{
			return null;
			}
		return $oQueryResult->fetch(\PDO::FETCH_ASSOC);
		}

	/**
	 * GenerateTaxOpaqueData - Generates Tax Opaque Data
	 *
	 * Tax 1 information
	 * TaxName=RoundRule=RoundGroup=TaxOtherTaxesFlag=LowTreshold=HighTreshold= AllowExemption=TaxID
	 *
	 * TaxName -  tax name
	 * RoundRule - tax rounding rule
	 * Reference to [TaxRules].[nTaxRulesID]
	 * RoundGroup- tax rounding group
	 * 	   0 = round tax per order line
	 * 	   1 = round tax per order item
	 * 	   2 = round tax per order
	 * TaxOtherTaxesFlag - whether tax is cumulative
	 * 	   0 = false
	 * 	   1 = true
	 * LowThreshold - low threshold (integer)
	 * HighThreshold- high threshold (integer)
	 * AllowExemption - whether the user exemption is allowed
	 * 	   0 = disallowed
	 * 	   1 = allowed
	 * TaxID	- tax ID
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @return string Tax Opaque Data
	 */
	public function GenerateTaxOpaqueData($sCountryName, $nTaxIndex)
		{
		if (null == $sCountryName)
			{
			return "";
			}
		$aTaxes = $this->GetCountryTax($sCountryName, $nTaxIndex);

		$sTaxName = $aTaxes['sTaxName'];
		$sTaxID = $aTaxes['nTaxID'];
		$nRoundRule = $aTaxes['nRoundingRule'];
		$sRoundGroup = $aTaxes['nRoundingGroup'];
		$nTaxOtherTaxesFlag = $aTaxes['bTaxOtherTaxes'];
		$nLowThreshold = $aTaxes['nRoundingGroup'];
		$nHighThreshold = $aTaxes['nHighThreshold'];
		$sAllowExemption = $aTaxes['bAllowTax' . $nTaxIndex . 'UserExemption'];

		$sTaxOpaqueData = "$sTaxName=$nRoundRule=$sRoundGroup=$nTaxOtherTaxesFlag=$nLowThreshold=$nHighThreshold=$sAllowExemption=$sTaxID=";
		return $sTaxOpaqueData;
		}

	/**
	 * GetTaxBands - Gets Tax Bands
	 *
	 * @access public
	 * @return string Tax Opaque Data
	 */
	public function GetTaxBands()
		{
		$sCountry = $this->GetCountry();
		$oQueryResult = $this->m_oCatalogDB->Select(
				'TaxBands', [['TaxBands', '*']], [['sCountryName', $sCountry]], [
			'join' => [
				['left', 'TaxZones', [[['TaxBands', 'nTaxID'], ['TaxZones', 'nTax1ID']]]],
				['left', 'TaxZoneMembers', [[['TaxZones', 'nZoneID'], ['TaxZoneMembers', 'nTaxZoneID']]]],
				['left', 'Countries', [[['TaxZoneMembers', 'nRegionID'], ['Countries', 'nCountryID']]]],
			],
				]
		);
		$aTaxBands = $oQueryResult->fetch(\PDO::FETCH_ASSOC);
		return $aTaxBands;
		}

	/**
	 * GenerateTaxOpaqueDataForOrderDetail - Generates Tax Opaque Data
	 *
	 * 
	 *   Shipping /Handling Tax opaque data
	 *   Band ID=Band Rate=Custom Rate=Band  Name=
	 *
	 *   Band ID               string for the band ID
	 *   Band Rate            string for the band rate
	 *   Custom Rate	string for the custom rate
	 *   Band Name          string for the band name
	 *
	 * @access public
	 * @param object $oTaxBand Tax Band
	 * @return string Tax Opaque Data
	 */
	public function GenerateTaxOpaqueDataForOrderDetail($oTaxBand)
		{
		return $oTaxBand->nBandID . "=" . $oTaxBand->dBandRate . "=" . $oTaxBand->nCustomRate . "=" . $this->m_aTaxBandNames[$oTaxBand->nBandID] . "=";
		}

	/**
	 * NormalizeTaxBand - Normalize Tax Band
	 *
	 * @access public
	 * @param array | object $vTaxBand Tax Band
	 * @return object Normalized Tax Band
	 */
	public function NormalizeTaxBand($vTaxBand)
		{
		if (is_array($vTaxBand))
			{
			$vTaxBand = (object) array_intersect_key($vTaxBand, array_flip(["nBandID", "dBandRate", "nCustomRate"]));
			$vTaxBand->nCustomRate = $vTaxBand->nCustomRate? : 0;
			}
		$vTaxBand->dBandRate = (double) $vTaxBand->dBandRate;
		return $vTaxBand;
		}

	/**
	 * ParseProductTaxData - Parses Product Tax Data
	 *
	 * @access public
	 * @param string $sTaxOpaqueData Tax Opaque Data
	 * @param arraz $aDefaultTaxBandAttributes Default Tax Band Attributes
	 * @return object Tax data
	 */
	public function ParseProductTaxData($sTaxOpaqueData, $aDefaultTaxBandAttributes = [])
		{
		$oDefault = (object) ["nBandID" => arr_get($aDefaultTaxBandAttributes, 'id', 0), "dBandRate" => float2sdnum(arr_get($aDefaultTaxBandAttributes, 'rate', 0.0)), "nCustomRate" => 0.0, "sName" => arr_get($aDefaultTaxBandAttributes, 'name', '')];
		if (empty($sTaxOpaqueData))
			{
			return $oDefault;
			}
		$aTaxOpaqueData = explode("=", $sTaxOpaqueData);
		array_pop($aTaxOpaqueData);
		if (count($aTaxOpaqueData) != 3)
			{
			return $oDefault;
			}
		$oTaxOpaqueData = (object) array_combine(["nBandID", "dBandRate", "nCustomRate"], $aTaxOpaqueData);
		return $oTaxOpaqueData;
		}

	/**
	 * LoadTaxes - Loads Taxes from DataBase
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @return array Tax Band Data
	 */
	public function LoadTaxes($sCountryName)
		{
		if (!isset($this->m_aTaxDefinition[$sCountryName]))
			{
			$this->m_aTaxDefinition[$sCountryName] = [
				'@attributes' =>
				[
					'taxinclusiveprices' => 'yes',
				],
			];
			$nTaxCount = 0;
			$aTaxes = [];
			$aTaxModel = $this->GetTaxModel();
			if (arr_get($aTaxModel, 'nModelID', TAX_MODEL_SIMPLE) == TAX_MODEL_SIMPLE)
				{
				$this->m_bSimpleTax = true;
				$aTaxes = $this->GetSimpleTaxes($aTaxModel);
				}
			else
				{
				$this->m_bSimpleTax = false;
				$aTaxes = $this->GetCountryTaxes($sCountryName);
				}
			foreach ($aTaxes as $aTax)
				{
				$nTaxCount++;
				$this->m_aTaxDefinition[$sCountryName]["tax$nTaxCount"]["@attributes"] = [
					'id' => $aTax['nTaxID'],
					'name' => iconv('WINDOWS-1256', 'UTF-8', $aTax['sTaxName']),
					'roundingmethod' => $this->m_aRoundingRules[$aTax['nRoundingRule']],
					'roundinglevel' => $this->m_aRoundingGroups[$aTax['nRoundingGroup']],
				];
				$oTaxBandsQueryResult = $this->m_oCatalogDB->Select('TaxBands', '*', [['nTaxID', $aTax['nTaxID']]]);
				if (false === $oTaxBandsQueryResult)
					{
					continue;
					}
				$aTaxBands = $oTaxBandsQueryResult->fetchAll();
				foreach ($aTaxBands as $nTaxBandIndex => $aTaxBand)
					{
//		   $aTaxBand['dBandRate'] = sdnum2float(float2sdnum($aTaxBand['dBandRate']));
					$nBandID = $aTaxBand['nBandID'];
					$sBandName = iconv('WINDOWS-1256', 'UTF-8', $aTaxBand['sBandName']);
					$this->m_aTaxBandNames[$nBandID] = $this->m_aTaxBandNames[$aTaxBand[$nBandID]] ? : $sBandName;
					$this->m_aTaxDefinition[$sCountryName]["tax$nTaxCount"]["band"][] = [
						'@attributes' => [
							'id' => $nBandID,
							'name' => $sBandName,
							'rate' => $aTaxBand['dBandRate'],
						],
					];
					}
				$this->m_aTaxBandNames[0] = $this->m_aTaxBandNames[0] ? : "Zero-Rated";
				$this->m_aTaxDefinition[$sCountryName]["tax$nTaxCount"]["band"][] = [
					'@attributes' => [
						'id' => 0,
						'name' => "Zero&amp;#45;Rated",
						'rate' => "0.000000",
					],
				];
				$this->m_aTaxBandNames[1] = $this->m_aTaxBandNames[1] ? : "Exempt";
				$this->m_aTaxDefinition[$sCountryName]["tax$nTaxCount"]["band"][] = [
					'@attributes' => [
						'id' => 1,
						'name' => "Exempt",
						'rate' => "0.000000",
					],
				];
				}
			$this->m_aTaxDefinition[$sCountryName]["@attributes"]["taxcount"] = $nTaxCount;
			}
		return $this->m_aTaxDefinition[$sCountryName];
		}

	/**
	 * TransformOrderToXMLArray - Transforms Order to XML Array
	 *
	 * @access public
	 * @param object $oOrder Order
	 * @return array Tax Band Data
	 */
	public function TransformOrderToXMLArray($oOrder)
		{
		$aProcessedOrder = $oOrder;
		return $aProcessedOrder;
		}

	/**
	 * AddCustomTaxToDefinition - Add Custom Tax To Definition
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @param int $nTaxIndex Tax Index
	 */
	public function AddCustomTaxToDefinition($sCountryName, $nTaxIndex)
		{
		$this->m_aTaxDefinition[$sCountryName]["tax$nTaxIndex"]["band"][] = [
			'@attributes' => [
				'id' => "c100-$nTaxIndex",
				'name' => "Custom tax 100%",
				'rate' => 100,
			],
		];
		$this->m_aTaxDefinition[$sCountryName]["tax$nTaxIndex"]["band"][] = [
			'@attributes' => [
				'id' => "c000-$nTaxIndex",
				'name' => "Custom tax 0%",
				'rate' => 0,
			],
		];
		}

	/**
	 * SetCustomTax - Sets Custom Tax
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @param int $nTaxIndex Tax Index
	 */
	public function SetCustomTax($sCountryName, $nTaxIndex, $aOrderDetailTaxXML, $nCustomRate)
		{
		$this->AddCustomTaxToDefinition($sCountryName, $nTaxIndex);
		$dRate = $nCustomRate / sdnum2float($aOrderDetailTaxXML['@attributes']['price']);
		if (isset($aOrderDetailTaxXML['lineband'][0]['@attributes']['applypercent']))
			{
			$nOtherTaxIndex = 3 - $nTaxIndex;
			$aOrderDetailTaxXML['lineband'][0]['@attributes']['tax' . $nTaxIndex . 'bandid'] = "c000-$nTaxIndex";
			$aOrderDetailTaxXML['lineband'][1]['@attributes']['applypercent'] -= $dRate;
			$aOrderDetailTaxXML['lineband'][1]['@attributes']['tax' . $nTaxIndex . 'bandid'] = "c000-$nTaxIndex";
			$aOrderDetailTaxXML['lineband'][2]['@attributes']['applypercent'] = $dRate;
			$aOrderDetailTaxXML['lineband'][2]['@attributes']['tax' . $nTaxIndex . 'bandid'] = "c100-$nTaxIndex";
			$aOrderDetailTaxXML['lineband'][2]['@attributes']['tax' . $nOtherTaxIndex . 'bandid'] = "c000-$nOtherTaxIndex";
			}
		else
			{
			$aOrderDetailTaxXML['lineband'][0]['@attributes']['applypercent'] = $dRate;
			$aOrderDetailTaxXML['lineband'][0]['@attributes']['tax' . $nTaxIndex . 'bandid'] = "c100-$nTaxIndex";
			$aOrderDetailTaxXML['lineband'][1]['@attributes']['applypercent'] = 100.00 - $dRate;
			$aOrderDetailTaxXML['lineband'][1]['@attributes']['tax' . $nTaxIndex . 'bandid'] = "c000-$nTaxIndex";
			}
		return $aOrderDetailTaxXML;
		}

	/**
	 * AddTaxBands - Adds Tax Bands
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @param array $aOrderProducts Order Products
	 * @param array $aOrderDetailTaxXMLs Order Detail Tax XML
	 * @return array Modified Order Detail Tax XML
	 */
	public function AddTaxBands($sCountryName, $aOrderProducts, $aOrderDetailTaxXMLs)
		{
		foreach ($aOrderDetailTaxXMLs as $nOrderDetailID => $aOrderDetailTaxXML)
			{
			$sProductReference = arr_get($aOrderDetailTaxXML, '@attributes.reference', 0);
			$aOrderProduct = arr_get($aOrderProducts, $sProductReference, []);
			$aOrderDetailTaxXMLs[$nOrderDetailID]['@attributes']['orderdetailid'] = $nOrderDetailID;
			$aTaxBands = $this->GetTaxBandsFromDefinition($sCountryName, $aOrderProduct);
			$aTaxBandIDs = $this->GetTaxBandIDs($aTaxBands);
			foreach ($aTaxBands as $nTaxIndex => $oTaxBand)
				{
				$aOrderDetailTaxXMLs[$nOrderDetailID]['@attributes']['tax' . $nTaxIndex . 'rate'] = (double) $oTaxBand->dBandRate;
				$aOrderDetailTaxXMLs[$nOrderDetailID]['@attributes']['tax' . $nTaxIndex . 'opaquedata'] = $this->GenerateTaxOpaqueDataForOrderDetail($oTaxBand);
				}
			switch ($this->CountCustom($aTaxBandIDs))
				{
				case 0:
					foreach ($aTaxBandIDs as $nTaxIndex => $nTaxBandID)
						{
						$aOrderDetailTaxXMLs[$nOrderDetailID]['@attributes']['tax' . $nTaxIndex . 'bandid'] = $nTaxBandID;
						}
					break;
				case 1:
					$nCustomTaxIndex = arr_get($aTaxBandIDs, 1, 0) == 6 ? 1 : 2;
					$nNonCustomTaxIndex = 3 - $nCustomTaxIndex;
					$nCustomRate = $aTaxBands[$nCustomTaxIndex]->nCustomRate;
					$nNonCustomTaxBandID = $aTaxBandIDs[$nNonCustomTaxIndex];
					$aOrderDetailTaxXMLs[$nOrderDetailID] = $this->SetCustomTax($sCountryName, $nCustomTaxIndex, $aOrderDetailTaxXMLs[$nOrderDetailID], $nCustomRate);
					$aOrderDetailTaxXMLs[$nOrderDetailID]['lineband'][0]['@attributes']['tax' . $nNonCustomTaxIndex . 'bandid'] = $nNonCustomTaxBandID;
					$aOrderDetailTaxXMLs[$nOrderDetailID]['lineband'][1]['@attributes']['tax' . $nNonCustomTaxIndex . 'bandid'] = $nNonCustomTaxBandID;
					break;
				case 2:
					foreach ($aTaxBands as $nTaxIndex => $oTaxBand)
						{
						$aOrderDetailTaxXMLs[$nOrderDetailID] = $this->SetCustomTax($sCountryName, $nTaxIndex, $aOrderDetailTaxXMLs[$nOrderDetailID], $oTaxBand->nCustomRate);
						}
					break;
				}
			}
		return $aOrderDetailTaxXMLs;
		}

	/**
	 * GetFirstTaxBand - Get First Tax Band
	 * 
	 * @access public
	 * @param string $sCountryName Country Name
	 * @param int $nTaxIndex Tax Index
	 * @return array Tax Band IDs
	 */
	public function GetFirstTaxBand($sCountryName, $nTaxIndex)
		{
		$aTaxBandsTmp = arr_get($this->m_aTaxDefinition, "$sCountryName.tax$nTaxIndex.band", []);
		$aTaxBandTmp = is_array($aTaxBandsTmp) ? array_shift($aTaxBandsTmp) : [];
		return arr_get($aTaxBandTmp, "@attributes", []);
		}

	/**
	 * GetTaxBandsFromDefinition - Get Tax Bands From Definition
	 * 
	 * @access public
	 * @param string $sCountryName Country Name
	 * @param array $aOrderProduct Order Product data
	 * @return array Tax Band IDs
	 */
	public function GetTaxBandsFromDefinition($sCountryName, $aOrderProduct)
		{
		$aTaxBands = [];
		$nTaxCount = arr_get($this->m_aTaxDefinition, "$sCountryName.@attributes.taxcount", 1);
		$nSpecTaxDiff = $this->m_bSimpleTax ? 0 : 2;
		for ($nTaxIndex = 1; $nTaxIndex <= $nTaxCount; $nTaxIndex++)
			{
			$nTaxOpaqueDataNo = arr_get($this->m_aTaxDefinition, "$sCountryName.tax$nTaxIndex.@attributes.id", $nTaxIndex + $nSpecTaxDiff) - $nSpecTaxDiff;
			$sTaxOpaqueData = arr_get($aOrderProduct, 'sTax' . $nTaxOpaqueDataNo . 'OpaqueData', '');
			$aDefaultTaxBandAttributes = $this->GetFirstTaxBand($sCountryName, $nTaxIndex);
			$aTaxBands[$nTaxIndex] = $this->ParseProductTaxData($sTaxOpaqueData, $aDefaultTaxBandAttributes);
			}
		return $aTaxBands;
		}

	/**
	 * GetTaxBandIDs - Get Tax Band IDs
	 *
	 * @access public
	 * @param array $aTaxBands Tax Bands
	 * @return array Tax Band IDs
	 */
	public function GetTaxBandIDs($aTaxBands)
		{
		return array_map(function($oTaxBand)
			{
			return $oTaxBand->nBandID;
			}, $aTaxBands);
		}

	/**
	 * GetShippingLine - Get Shipping Line
	 *
	 * @access public
	 * @param double $dShipping Shipping
	 * @return array Shipping Order Line
	 */
	public function GetShippingLine($dShipping, $sCountryName)
		{
		$aShipping = [
			'@attributes' =>
			[
				'id' => 'shipping',
				'price' => $dShipping,
				'qty' => 1,
				'groupid' => 'discountedcart',
			]
		];
		// <orderline id="shipping" price="350.000000" qty="1" tax1bandid="301shipping"/>
		// <orderline id="shipping" price="1130.000000" qty="1" tax1bandid="301" tax2bandid="402"/>
		// TODO!!
		if ($this->m_bSimpleTax)
			{
			$nTaxModel = TAX_MODEL_SIMPLE;
			$aTaxes = $this->GetSimpleTaxes($this->GetTaxModel());
			}
		else
			{
			$nTaxModel = TAX_MODEL_ADVANCED;
			$aTaxes = $this->GetCountryTaxes($sCountryName);
			}
		$nTaxBy = TAX_BY_ALWAYS_TAX;
		$nTaxCount = $this->m_aTaxDefinition[$sCountryName]["@attributes"]["taxcount"];
		$sShippingTax1OpaqueData = arr_get($aTaxes, '1.sShippingOpaqueData', "0=0=0=");
		$oTaxOpaqueData = $this->ParseProductTaxData($sShippingTax1OpaqueData);
		if ($oTaxOpaqueData->nBandID != 5) // Pro-Rata
			{
			$aShipping['@attributes']['tax1bandid'] = $oTaxOpaqueData->nBandID;
			}
		if ($nTaxCount > 1)
			{
			$sShippingTax2OpaqueData = arr_get($aTaxes, '2.sShippingOpaqueData', "0=0=0=");
			$oTaxOpaqueData = $this->ParseProductTaxData($sShippingTax2OpaqueData);
			if ($oTaxOpaqueData->nBandID != 5) // Pro-Rata
				{
				$aShipping['@attributes']['tax2bandid'] = $oTaxOpaqueData->nBandID;
				}
			}
		return $aShipping;
		}

	/**
	 * CountCustom - Count Custom
	 *
	 * @access public
	 * @param array $aTaxBandIDs Tax Band IDs
	 * @return int No of Custom Taxes
	 */
	public function CountCustom($aTaxBandIDs)
		{
		$aCountValues = array_count_values($aTaxBandIDs);
		return arr_get($aCountValues, 6, 0);
		}

	/**
	 * ProcessTaxes - Processing Taxes (Class Entry Point)
	 *
	 * @access public
	 * @param string $sCountryName Country Name
	 * @param array $aOrderProducts Order Products
	 * @param array $aOrderDetailTaxXMLs Order Detail Tax XML
	 * @param double $dShipping Shipping
	 * @return 
	 */
	public function ProcessTaxes($sCountryName, $aOrderProducts, $aOrderDetailTaxXMLs, $dShipping = 0)
		{
		if (!class_exists('CTaxModule'))
			{
			throw new \SDExtension\SDException\SDException("CTaxModule not found");
			}
		$this->LoadTaxes($sCountryName); // $this->m_aTaxDefinition[$sCountryName]
		$aCustomer = ['@attributes' => ['tax1exempt' => 'no',]];
		$aZone = ['@attributes' => ['tax1applies' => 'yes',]];
		$nTaxCount = $this->m_aTaxDefinition[$sCountryName]["@attributes"]["taxcount"];
		if ($nTaxCount > 1)
			{
			$aCustomer['@attributes']['tax2exempt'] = 'no';
			$aZone['@attributes']['tax2applies'] = 'yes';
			}
		$aOrderDetailTaxXMLs = $this->AddTaxBands($sCountryName, $aOrderProducts, $aOrderDetailTaxXMLs);
		$aOrderDetailTaxXMLs = [
			'group' => [
				'@attributes' => ['id' => 'discountedcart'],
				'group' => [
					'@attributes' => ['id' => 'cart'],
					'orderline' => $aOrderDetailTaxXMLs,
				]
			]
		];
		$aOrderDetailTaxXMLs['orderline'] = $this->GetShippingLine($dShipping, $sCountryName);
		$oTaxModule = new \CTaxModule();
		$aMessage = [
			'taxdefinition' => $this->m_aTaxDefinition[$sCountryName],
			'customer' => $aCustomer,
			'zone' => $aZone,
			'order' => $aOrderDetailTaxXMLs,
		];
		$oTaxXML = \SDExtension\Helper\Array2XML::createXML("message", $aMessage);
		$sTaxXML = $oTaxXML->saveXML();
		$sTaxModuleOutputXML = $oTaxModule->Process($sTaxXML);
		$aTaxModuleOutputXML = \SDExtension\Helper\XML2Array::createArray($sTaxModuleOutputXML);
		if (arr_get($aTaxModuleOutputXML, 'MESSAGE.ORDER.GROUP.GROUP.ORDERLINE.LINEBAND', null) == null)
			{
			foreach (arr_get($aTaxModuleOutputXML, 'MESSAGE.ORDER.GROUP.GROUP.ORDERLINE', []) as $nKey => $aOrderLine)
				{
				$aTaxModuleOutputXML['MESSAGE']['ORDER']['GROUP']['GROUP']['ORDERLINE']["tmp$nKey"] = $aOrderLine;
				unset($aTaxModuleOutputXML['MESSAGE']['ORDER']['GROUP']['GROUP']['ORDERLINE'][$nKey]);
				}
			foreach (arr_get($aTaxModuleOutputXML, 'MESSAGE.ORDER.GROUP.GROUP.ORDERLINE', []) as $sKey => $aOrderLine)
				{
				$aTaxModuleOutputXML['MESSAGE']['ORDER']['GROUP']['GROUP']['ORDERLINE'][arr_get($aOrderLine, '@attributes.ORDERDETAILID', 0)] = $aOrderLine;
				unset($aTaxModuleOutputXML['MESSAGE']['ORDER']['GROUP']['GROUP']['ORDERLINE'][$sKey]);
				}
			}
		else
			{
			$aOrderLine = arr_get($aTaxModuleOutputXML, 'MESSAGE.ORDER.GROUP.GROUP.ORDERLINE', []);
			$nOrderDetailID = arr_get($aTaxModuleOutputXML, 'MESSAGE.ORDER.GROUP.GROUP.ORDERLINE.@attributes.ORDERDETAILID', []);
			unset($aTaxModuleOutputXML['MESSAGE']['ORDER']['GROUP']['GROUP']['ORDERLINE']);
			$aTaxModuleOutputXML['MESSAGE']['ORDER']['GROUP']['GROUP']['ORDERLINE'][$nOrderDetailID] = $aOrderLine;
			}
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug("TaxModule REQUEST: " . $sTaxXML);
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug("TaxModule RESPONSE: " . str_replace("\t\t", "  ", $sTaxModuleOutputXML));
		return $aTaxModuleOutputXML;
		}

	}
