<?php

/**
 * CDataBaseAccessNoPDO.php - Implementation file for Ms Access DataBase class using ODBC.
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

namespace SDExtension\DB;

class CDataBaseAccessNoPDO extends CDataBaseAccess
	{

	/**
	 * __construct - Object constructor method
	 *
	 * @access public
	 * @param array $aConnectionDetails
	 * @return void
	 */
	public function __construct($aConnectionDetails, $sDatabaseID)
		{
		\SDExtension\Helper\CLogger::get()->addDebug(__CLASS__);
		try
			{
			$sDsn = arr_get($aConnectionDetails, 'sDsn', '');
			$this->m_sDBUser = arr_get($aConnectionDetails, 'sUser', '');
			$sPassword = arr_get($aConnectionDetails, 'sPassword', '');
			$this->m_sDataBaseID = arr_get($aConnectionDetails, 'sDatabaseID', '');
			$this->m_aDataBases = arr_get($aConnectionDetails, 'aDataBases', []);
			$this->m_oDB = new CMockAccessOdbcPdo($sDsn, $this->m_sDBUser, $sPassword);
			\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addInfo("Connected to Database: $sDatabaseID");
			}
		catch (\Exception $e)
			{
			echo $e->getMessage();
			}
		}

	}

/**
 * CMockAccessOdbcPdo - Mock PDO Class to using ODBC
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */
class CMockAccessOdbcPdo
	{

	/**
	 * @var object $m_oConnection Connection
	 */
	protected $m_oConnection;

	/**
	 * @var object $m_oResult Result
	 */
	protected $m_oResult = null;

	/**
	 * __construct - Object constructor method
	 *
	 * @access public
	 * @param string $sDsn DSN
	 * @param string $sDBUser DB User
	 * @param string $sPassword Password
	 * @return void
	 */
	public function __construct($sDsn, $sDBUser, $sPassword)
		{
		$this->m_oConnection = odbc_connect($sDsn, $sDBUser, $sPassword);
		}

	/**
	 * prepare - Prepare query
	 *
	 * @access public
	 * @param string $sQuery Query
	 * @return CMockAccessOdbcPdoStatement
	 */
	public function prepare($sQuery)
		{
		return new CMockAccessOdbcPdoStatement($this->m_oConnection, $sQuery);
		}

	/**
	 * tables - Getting tables
	 *
	 * @access public
	 * @return resource|bool An ODBC result identifier containing table information or FALSE on failure.
	 */
	public function tables()
		{
		return odbc_tables($this->m_oConnection);
		}

	/**
	 * tableExists - Does table exist?
	 *
	 * @access public
	 * @param string $sTableName Table Name
	 * @return bool Table exist
	 */
	public function tableExists($sTableName)
		{
		$oResult = $this->tables();
		$bFound = false;
		while (!$bFound && $aResult = odbc_fetch_array($oResult))
			{
			$bFound = $sTableName == $aResult["TABLE_NAME"];
			}
		return $bFound;
		}

	/**
	 * lastInsertId - Last Insert ID
	 *
	 * @access public
	 * @return mixed Last Insert ID
	 */
	public function lastInsertId()
		{
		$oStatement = odbc_prepare($this->m_oConnection, "SELECT LAST_INSERT_ID()");
		odbc_execute($oStatement);
		$oResource = odbc_fetch_array($oStatement);
		return $oResource['LAST_INSERT_ID'];
		}

	/**
	 * exec - Execute query
	 *
	 * @access public
	 * @param string $sStatement Statement
	 * @return int|bool Number of rows or FALSE on failure
	 */
	public function exec($sStatement)
		{
		$oResult = odbc_exec($this->m_oConnection, $sStatement);
		return $oResult === false ? false : odbc_num_rows($oResult);
		}

	/**
	 * query - Execute query
	 *
	 * @access public
	 * @param string $sStatement Statement
	 * @param int $nReturnType Return Type
	 * @return CMockAccessOdbcPdo This object
	 */
	public function query($sStatement, $nReturnType = \PDO::FETCH_ASSOC)
		{
		$this->m_oResult = odbc_exec($this->m_oConnection, $sStatement);
		return $this;
		}

	/**
	 * fetch - Fetch result array
	 *
	 * @access public
	 * @param string $sFetchStyle Fetch Style
	 * @param string $sCursorOrientation Cursor Orientation
	 * @param int $nCursorOffset Cursor Offset
	 * @return array|bool An array that corresponds to the fetched row, or FALSE if there are no more rows.
	 */
	public function fetch($sFetchStyle = null, $sCursorOrientation = \PDO::FETCH_ORI_NEXT, $nCursorOffset = 0)
		{
		return odbc_fetch_array($this->m_oResult);
		}

	/**
	 * fetchAll - Fetch all results
	 *
	 * @access public
	 * @param string $sFetchStyle Fetch Style
	 * @param string $sFetchArgument Fetch Argument
	 * @param array $aCtorArgs Ctor Args
	 * @return array Array of results
	 */
	public function fetchAll($sFetchStyle = null, $sFetchArgument = null, array $aCtorArgs = [])
		{
		$aResultSet = [];
		while ($aResult = odbc_fetch_array($this->m_oResult))
			{
			$aResultSet[] = $aResult;
			}
		return $aResultSet;
		}

	}

/**
 * CMockAccessOdbcPdoStatement - Mock PDO Statement Class to using ODBC
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */
class CMockAccessOdbcPdoStatement
	{

	/**
	 * @var string $m_sQuery Query
	 */
	protected $m_sQuery;

	/**
	 * @var array $m_aParam Param
	 */
	protected $m_aParams;

	/**
	 * @var object $m_oStatement Statement
	 */
	protected $m_oStatement;

	/**
	 * __construct - Object constructor method
	 *
	 * @access public
	 * @param object $oConnection Connection
	 * @param string $sQuery Query
	 * @return void
	 */
	public function __construct($oConnection, $sQuery)
		{
//	$this->m_sQuery = preg_replace('/(?<=\s|^):[^\s:]++/um', '?', $sQuery);
		$this->m_sQuery = $sQuery;
		$this->m_aParams = null;
		$this->extractParam($sQuery);
		$this->m_oStatement = odbc_prepare($oConnection, $this->m_sQuery);
		}

	/**
	 * bindValue - Bind Value
	 *
	 * @access public
	 * @param string $sParam Parameter
	 * @param mixed $vValue Value
	 * @return void
	 */
	public function bindValue($sParam, $vValue)
		{
		$this->m_aParams[$sParam] = $vValue;
		}

	/**
	 * execute - Execute
	 *
	 * @access public
	 * @return void
	 */
	public function execute()
		{
		if ($this->m_aParams == null)
			{
			odbc_execute($this->m_oStatement);
			}
		else
			{
			odbc_execute($this->m_oStatement, $this->m_aParams);
			}
		$this->clearParam();
		}

	/**
	 * fetch - Fetch
	 *
	 * @access public
	 * @param mixed $vOption Options
	 * @return array|bool An array that corresponds to the fetched row, or FALSE if there are no more rows.
	 */
	public function fetch($vOption = null)
		{
		return odbc_fetch_array($this->m_oStatement);
		}

	/**
	 * extractParam - Extract Parameters
	 *
	 * @access protected
	 * @param mixed $sQuery Query
	 * @return void
	 */
	protected function extractParam($sQuery)
		{
		$aQueries = explode(" ", $sQuery);
		$nIndex = 0;

		while (isset($aQueries[$nIndex]))
			{
			if (preg_match("/^:/", $aQueries[$nIndex]))
				{
				$this->m_aParams[$aQueries[$nIndex]] = null;
				}
			++$nIndex;
			}
		}

	/**
	 * clearParam - Clear Parameters
	 *
	 * @access protected
	 * @return void
	 */
	protected function clearParam()
		{
		$nIndex = 0;

		while (isset($this->m_aParams[$nIndex]))
			{
			$this->m_aParams[$nIndex] = null;
			++$nIndex;
			}
		}

	}
