<?php

/**
 * CDataBaseSqlServer.php - Implementation file for SQL Server DataBase class.
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

namespace SDExtension\DB;

class CDataBaseSqlServer extends CDataBase
	{

	/**
	 * @var string $m_sColumnQuoteBegin Begin quote sign for DB Columns
	 */
	protected $m_sColumnQuoteBegin = "[";

	/**
	 * @var string $m_sColumnQuoteEnd End quote sign for DB Columns
	 */
	protected $m_sColumnQuoteEnd = "]";

	/**
	 * @var array $m_aCatalogTables Catalog Tables
	 */
	protected $m_aCatalogTables = [
		"Additional Files",
		"Address",
		"AlsoBought",
		"BestSellers",
		"BrochureFragments",
		"BrochurePages",
		"Buyer",
		"Catalog",
		"Catalog section",
		"Checkout Phase",
		"Checkout Prompt",
		"ColorScheme",
		"Configuration",
		"ContentCategories",
		"ContentLists",
		"Countries",
		"Coupons",
		"CouponSummaryActivity",
		"CreditCard",
		"currencies",
		"Customer",
		"CustomerCardDetails",
		"DesignObjects",
		"DesignObjectTypes",
		"DesignProperties",
		"DesignPropertyGroup",
		"DiscountProductGroups",
		"DiscountRestrictionList",
		"Discounts",
		"DiscountTriggers",
		"DuplicatedFields",
		"ECMachine",
		"ECUser",
		"EPOSPriceMapping",
		"EPOSTaxMapping",
		"FileInfo",
		"FileList",
		"Formatting",
		"FragmentItemsList",
		"GFSPackageDimension",
		"ImportMappings",
		"InitialisationStatus",
		"LandlordAdjustment",
		"Marketing Text",
		"MergeFileState",
		"ML_TEMP_PRINTING_TABLE",
		"NewProducts",
		"OCCCurrencies",
		"OCCOptionalCurrencies",
		"OCCProvider",
		"OCCProviderAttributes",
		"OCCVersions",
		"Order",
		"OrderColor",
		"OrderDetail",
		"OrderHistory",
		"OrderMail",
		"OrderStatusMessage",
		"OrderTracking",
		"OutstandingCouponChanges",
		"OutstandingOrderChanges",
		"OutstandingStockChanges",
		"PageTypes",
		"PaymentHistory",
		"PaymentMethodLocations",
		"PaymentMethodRegions",
		"PaymentMethods",
		"PaymentRegionLocations",
		"PaymentRegions",
		"Person",
		"PreprocessedTaxTable",
		"Price Schedules",
		"Prices",
		"Product",
		"ProductLinkInfo",
		"ProductProperties",
		"PropertyTabs",
		"RelatedProducts",
		"SearchAttributeChoicesOrder",
		"SearchGeneralFilterGroups",
		"SearchPriceBand",
		"SearchSortOrders",
		"Section Classifications",
		"SettlementTime",
		"Setup",
		"Setup2",
		"Setup3",
		"SetupInfo",
		"SSPOnlineErrorData",
		"SSPProvider",
		"SSPProviderClasses",
		"SSPProviderFeature",
		"SSPProviderFeatureSetup",
		"SSPProviderVersions",
		"States",
		"TaxBands",
		"Taxes",
		"TaxModels",
		"TaxOrderOpaqueData",
		"TaxSetup",
		"TaxZoneMembers",
		"TaxZones",
		"TrueFalseValues",
		"UserDefinedProperties",
		"Variable",
		"WarningStatus",
	];

	/**
	 * QuoteTable - Quotes Table name
	 *
	 * @access public
	 * @param string $sTableName Table name
	 * @return string Quoted Table name
	 */
	public function QuoteTable($sTableName)
		{
		$sDataBaseID = arr_get($this->GetDataBases(), $this->GetDatabaseID(), null);
		return $this->QuoteColumn($sDataBaseID) . "." .
			$this->QuoteColumn(DB_COMMON_DBO) . "." .
			$this->QuoteColumn($sTableName);
		}

	/**
	 * BasicInsert - Generates and runs Basic INSERT expression
	 *
	 * @access protected
	 * @param string $sTable Table name
	 * @param array $aValues Array of (DB Column => Value) pairs
	 * @return mixed Last inserted ID
	 */
	protected function BasicInsert($sTable, $aValues)
		{
		$sTableQuoted = $this->QuoteTable($sTable);
		$sColumnList = $this->QuoteColumnList(array_keys($aValues));
		$sValueList = $this->QuoteValueList(array_values($aValues));
		$sQuery = "INSERT INTO $sTableQuoted (" . $sColumnList . ") VALUES (" . $sValueList . ");";
		$nResult = $this->m_oDB->exec($sQuery);
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug(__FUNCTION__ . " : $sQuery");
		return $this->m_oDB->lastInsertId();
		}

	/**
	 * Sanitize - Sanitizes string
	 *
	 * @access protected
	 * @param string $sData Data to sanitize
	 * @return string Sanitized string
	 */
	protected function Sanitize($sData)
		{
		return str_replace("'", "''", $sData);
		}

	/**
	 * CompileSelectSql - Compiles SQL expression
	 * 
	 * @access protected
	 * @param array $aWhere Array of conditions
	 * @return string WHERE statement
	 */
	protected function CompileSelectSql($aQuery = [])
		{
		$sSelectSQL = "SELECT";
		if (!empty($aQuery["limit"]))
			{
			$sSelectSQL .= " TOP $aQuery[limit] ";
			}
		if (!empty($aQuery["select"]))
			{
			$sSelectSQL .= " $aQuery[select] ";
			}
		else
			{
			$sSelectSQL .= " * ";
			}
		if (!empty($aQuery["from"]))
			{
			$sSelectSQL .= "FROM ";
			if (!empty($aQuery["join"]))
				{
				$sSelectSQL .= "($aQuery[from] $aQuery[join])";
				}
			else
				{
				$sSelectSQL .= "$aQuery[from] ";
				}
			}
		if (!empty($aQuery["where"]))
			{
			$sSelectSQL .= "WHERE $aQuery[where] ";
			}
		if (!empty($aQuery["group_by"]))
			{
			$sSelectSQL .= "GROUP BY $aQuery[group_by] ";
			}
		if (!empty($aQuery["order_by"]))
			{
			$sSelectSQL .= "ORDER BY $aQuery[order_by] ";
			}
		return $sSelectSQL;
		}

	/**
	 * BoolValue - Converts Boolean to appropriate value depending on driver
	 * 
	 * @access protected
	 * @param bool $bData Boolean value
	 * @return int
	 */
	protected function BoolValue($bData)
		{
		return $bData ? 1 : 0;
		}

	}
