<?php

/**
 * CTable.php - Table ancestor class implementation.
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

namespace SDExtension\DB\Table;

class CTable
	{

	/**
	 * @var object DataBase Object
	 */
	protected $m_oDB = null;

	/**
	 * @var string Table name
	 */
	protected $m_sTable = 'table';

	/**
	 * @var string Table ID Column
	 * @todo Does not support compound keys
	 */
	protected $m_sIdColumn = 'id';

	/**
	 * @var bool Is ID autogenerated?
	 */
	protected $m_bAutoID = true;

	/**
	 * @var array Record Default values
	 */
	protected $m_aDefaultValues = [];

	/**
	 * @var array Record Actual values
	 */
	protected $m_aActualValues = [];

	/**
	 * @var array Record validation rules
	 */
	protected $m_aRules = [];

	/**
	 * @var bool Is New Record?
	 */
	protected $m_bNewRecord;

	/**
	 * @var bool Is Modified?
	 */
	protected $m_bModified;

	/**
	 * __construct - Object constructor method
	 *
	 * @access public
	 * @param CDataBase $oDB Database Object
	 * @return void
	 */
	public function __construct(\SDExtension\DB\CDataBase $oDB)
		{
		$this->m_oDB = $oDB;
		$this->Reset();
		}

	/**
	 * GetDB - DB
	 *
	 * @access public
	 * @return object DB
	 */
	public function GetDB()
		{
		return $this->m_oDB->GetDB();
		}

	/**
	 * Reset - Resets class variables
	 *
	 * @access public
	 * @return void
	 */
	public function Reset()
		{
		$this->m_bNewRecord = true;
		$this->m_bModified = false;
		$this->m_aActualValues = $this->m_aDefaultValues;
		}

	/**
	 * Save - Saves (inserts or updates) actual record to DB
	 *
	 * @access public
	 * @param array|null $aActualValues Actual Values
	 * @return void
	 */
	public function Save($aActualValues = null)
		{
		if ($aActualValues == null)
			{
			$aActualValues = $this->m_aActualValues;
			}
		elseif (is_array($aActualValues))
			{
			$this->m_aActualValues = $aActualValues;
			}
		$sID = arr_get($aActualValues, $this->m_sIdColumn, '');
		if ($this->m_bModified)
			{
			if ($this->m_bNewRecord)
				{
				if (empty($sID))
					{
					unset($aActualValues[$this->m_sIdColumn]);
					}
				$sID = $this->m_oDB->Insert($this->m_sTable, $aActualValues, $this->m_bAutoID, $this->m_sIdColumn);
				}
			else
				{
				unset($aActualValues[$this->m_sIdColumn]);
				$this->m_oDB->Update($this->m_sTable, $aActualValues, $this->GenerateWhereID($sID));
				}
			$this->m_bModified = false;
			}
		return $sID;
		}

	/**
	 * UpdateSave - Updates record and saves it to the DB
	 *
	 * @access public
	 * @param array $aActualValues Values to set
	 * @param bool $bModified Is modified?
	 * @param bool $bAutoId Get id from Actual Record
	 * @return array Actual record values
	 */
	public function UpdateSave($aActualValues = [], $bModified = true, $bAutoId = false)
		{
		$vID = arr_get($bAutoId ? $this->m_aActualValues : $aActualValues, $this->m_sIdColumn, '');
		unset($aActualValues[$this->m_sIdColumn]);
		$this->m_oDB->Update($this->m_sTable, $aActualValues, $this->GenerateWhereID($vID));
		$aActualValues[$this->m_sIdColumn] = $vID;
		$this->Set($aActualValues, $bModified);
		}

	/**
	 * Set - Sets Active record values
	 *
	 * @access public
	 * @param array $aActualValues Values to set
	 * @param bool $bModified Is modified?
	 * @return array Actual record values
	 */
	public function Set($aActualValues = [], $bModified = true)
		{
		$this->m_bModified = $bModified;
		$this->m_aActualValues = array_merge($this->m_aActualValues, $aActualValues);
		return $this->m_aActualValues;
		}

	/**
	 * Get - Gets Active record values
	 *
	 * @access public
	 * @return array Actual record values
	 */
	public function Get()
		{
		return $this->m_aActualValues;
		}

	/**
	 * GetRecords - Gets all records from the DB
	 *
	 * @access public
	 * @return GeneratorObject
	 */
	public function GetRecords()
		{
		foreach ($this->m_oDB->Select($this->m_sTable) as $row)
			{
			yield $row;
			}
		}

	/**
	 * Load - Loads one record from the DB (the first record according to the condition)
	 *
	 * @access public
	 * @param array $aWhere Conditions
	 * @param int $nReturnType Return Type (associative array by default)
	 * @return array|object Record
	 */
	public function Load($aWhere = [], $nReturnType = \PDO::FETCH_ASSOC)
		{
		$oQueryResult = $this->m_oDB->Select($this->m_sTable, "*", $aWhere);
		if ($oQueryResult === false)
			{
			return [];
			}
		$this->m_bNewRecord = false;
		$vRecord = $oQueryResult->fetch($nReturnType);
		if ($vRecord === false)
			{
			return [];
			}
		return $this->Set($vRecord, false);
		}

	/**
	 * LoadByValue - Loads one record by condition: Column = Value
	 *
	 * @access public
	 * @param string $sColumn Column name
	 * @param string $sValue Column value
	 * @return array Record
	 */
	public function LoadByValue($sColumn, $sValue)
		{
		$aWhere = $this->GenerateWhereCondition($sColumn, $sValue);
		return $this->Load($aWhere);
		}

	/**
	 * LoadByID - Loads one record by given ID value
	 *
	 * @access public
	 * @param string $sValue Column value
	 * @return array Record
	 */
	public function LoadByID($sValue)
		{
		$aWhere = $this->GenerateWhereID($sValue);
		return $this->Load($aWhere);
		}

	/**
	 * Reload - Reloads Actual record, or loads a new one by given ID value 
	 *
	 * @access public
	 * @param string $sValue Column value
	 * @return array Record
	 */
	public function Reload($sID = null)
		{
		if ($sID == null)
			{
			$sID = $this->m_aActualValues[$this->m_sIdColumn];
			}
		return $this->LoadByID($sID);
		}

	/**
	 * GenerateWhereCondition - Generates WHERE Condition
	 *
	 * @access private
	 * @param string $sColumn Column name
	 * @param string $sValue Column value
	 * @return array Where condition array
	 */
	private function GenerateWhereCondition($sColumn, $sValue)
		{
		return [[$sColumn, "=", $sValue]];
		}

	/**
	 * GenerateWhereCondition - Generates WHERE Condition with given ID value 
	 *
	 * @access private
	 * @param string $sValue ID Column value
	 * @return array Where condition array
	 */
	private function GenerateWhereID($sValue)
		{
		return $this->GenerateWhereCondition($this->m_sIdColumn, $sValue);
		}

	/**
	 * CreateOriginalCopies - Creates Original Copies
	 *
	 * @access public
	 * @return void
	 */
	public function CreateOriginalCopies()
		{
		foreach ($this->m_aActualValues as $sKey => $sValue)
			{
			foreach (array("Original ", "dOriginal", "sOriginal", "sOrig") as $sOriginalPrefix)
				{
				$sAltKey = substr($sKey, 1);
				if (isset($this->m_aDefaultValues["$sOriginalPrefix$sKey"]))
					{
					$this->m_aActualValues["$sOriginalPrefix$sKey"] = $sValue;
					}
				elseif (isset($this->m_aDefaultValues["$sOriginalPrefix$sAltKey"]))
					{
					$this->m_aActualValues["$sOriginalPrefix$sAltKey"] = $sValue;
					}
				}
			}
		}

	/**
	 * Validate - Validates record
	 *
	 * @access public
	 * @param array $aActualValues Column value
	 * @return mixed true | Array of errors
	 */
	public function Validate($aActualValues = [])
		{
		$oValidationResult = \SimpleValidator\Validator::validate($aActualValues, $this->GetRules());
		$oValidationResult->customErrors(load_config(CONFIG_APP_ROOT . "validator/en.php"));
		if ($oValidationResult->isSuccess() == true)
			{
			return true;
			}
		else
			{
			return $oValidationResult->getErrors();
			}
		}

	/**
	 * GetRules - Get Validator rules
	 *
	 * @access protected
	 * @return array Rules
	 */
	protected function GetRules()
		{
		return $this->m_aRules;
		}

	}
