<?php

/**
 * Tools.php - Contains various string and array helper functions
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

/**
 * Checks if array is associative
 *
 * @param   array    $array    an array
 * @return  bool  Is array associative?
 */
function is_assoc($array)
	{
	return (bool) count(array_filter(array_keys($array), 'is_string'));
	}

/**
 * Checks if varible is a string, or array. Array will be imploded to string.
 *
 * @param   string|array    $vData    string or an array
 * @return  string  original string, or imploded array
 */
function string_or_implode_array($vData)
	{
	if (is_array($vData))
		{
		return implode(' ', $vData);
		}
	return (string) $vData;
	}

/**
 * Builds XML from array
 *
 * @todo Generates tags onlywithout attributes
 * @param   array   $aArray    An array containing XML nodes
 * @param   string  $sRoot    XML root tag
 * @param   string  $sCustomHeader    Custom header tag
 * @return  string  XML
 */
function array2xml($aArray, $sRoot = 'root', $sCustomHeader = '<?xml version="1.0" encoding="UTF-8"?>')
	{
	$sRootNode = "$sCustomHeader<$sRoot/>";
	$oXML = new \SimpleXMLElement($sRootNode);
	array2xml_recursive($aArray, $oXML, null);
	$sData = $oXML->asXML();
	return $sData;
	}

/**
 * Builds XML from array (helper function, recursive part)
 *
 * @param   array   $aArray    An array containing XML nodes
 * @param   SimpleXMLElement  $oSimpleXMLElement    XML root tag
 * @return  void
 */
function array2xml_recursive($aArray, &$oSimpleXMLElement)
	{
	foreach ($aArray as $sNodeName => $vValue)
		{
		if (is_array($vValue))
			{
			if (is_assoc($vValue))
				{
				$oXML = $oSimpleXMLElement->addChild($sNodeName);
				array2xml_recursive($vValue, $oXML);
				}
			else
				{
				foreach ($vValue as $vValue2)
					{
					if (is_array($vValue2))
						{
						$oXML = $oSimpleXMLElement->addChild($sNodeName);
						array2xml_recursive($vValue2, $oXML);
						}
					else
						{
						$oSimpleXMLElement->addChild($sNodeName, $vValue2);
						}
					}
				}
			}
		else
			{
			$oSimpleXMLElement->addChild($sNodeName, $vValue);
			}
		}
	}

/**
 * Converts XML string to JSON string
 *
 * @param   string   $sXMLString    XML
 * @return  string JSON
 */
function xml2json($sXMLString)
	{
	$oXML = simplexml_load_string($sXMLString);
	$sJSON = json_encode($oXML);
	return $sJSON;
	}

/**
 * Converts XML string to PHP array
 *
 * @param   string   $sXMLString    XML
 * @return  array
 */
function xml2array($sXMLString)
	{
	$sJSON = xml2json($sXMLString);
	$aArray = json_decode($sJSON, TRUE);
	return $aArray;
	}

/**
 * Gets a dot-notated key from an array, with a default value if it does
 * not exist.
 *
 * @param   array    $array        The search array
 * @param   mixed    $nKey          The dot-notated key or array of keys
 * @param   string   $default      The default value
 * @param   bool     $bCheckEmpty  Checks if value is not empty, returns default value, if it is empty
 * @return  mixed
 */
function arr_get($array, $nKey, $default = null, $bCheckEmpty = false)
	{
	if (!is_array($array) and ! $array instanceof \ArrayAccess)
		{
		return $default;
//	throw new \InvalidArgumentException('First parameter must be an array or ArrayAccess object.');
		}

	if (is_null($nKey))
		{
		return $array;
		}

	if (is_array($nKey))
		{
		$return = array();
		foreach ($nKey as $k)
			{
			$return[$k] = arr_get($array, $k, $default);
			}
		return $return;
		}

	is_object($nKey) and $nKey = (string) $nKey;

	if (array_key_exists($nKey, $array))
		{
		if ($bCheckEmpty && (empty($array[$nKey]) || (is_string($array[$nKey]) && empty(trim($array[$nKey])))))
			{
			return $default;
			}
		return $array[$nKey];
		}

	foreach (explode('.', $nKey) as $nKey_part)
		{
		if ((is_a($array, 'ArrayAccess') and isset($array[$nKey_part])) === false)
			{
			if (!is_array($array) or ! array_key_exists($nKey_part, $array))
				{
				return $default;
				}
			}

		$array = $array[$nKey_part];
		}

	if ($bCheckEmpty && empty($array))
		{
		return $default;
		}
	return $array;
	}

/**
 * Checks if string starts with prefix
 *
 * @param   string   $sHaystack   The string to search in
 * @param   string   $sNeedle   The string to search for
 * @return  bool String starts with prefix
 */
function starts_with($sHaystack, $sNeedle)
	{
// search backwards starting from haystack length characters from the end
	return $sNeedle === "" || strrpos($sHaystack, $sNeedle, -strlen($sHaystack)) !== FALSE;
	}

/**
 * Checks if string ends with suffix
 *
 * @param   string   $sHaystack   The string to search in
 * @param   string   $sNeedle   The string to search for
 * @return  bool String ends with suffix
 */
function ends_with($sHaystack, $sNeedle)
	{
// search forward starting from end minus needle length characters
	return $sNeedle === "" || (($sTemp = strlen($sHaystack) - strlen($sNeedle)) >= 0 && strpos($sHaystack, $sNeedle, $sTemp) !== FALSE);
	}

/**
 * Removes string prefix
 *
 * @param   string   $sStr   The string to search in
 * @param   string   $sPrefix   Prefix
 * @return  string String without prefix
 */
function remove_prefix($sStr, $sPrefix)
	{
	return substr($sStr, strlen($sPrefix));
	}

/**
 * Removes string prefix
 *
 * @param   string   $sStr   The string to search in
 * @param   string   $sPrefix   Prefix
 * @return  string String without prefix
 */
function remove_suffix($sStr, $sPrefix)
	{
	return substr($sStr, 0, -1 * strlen($sPrefix));
	}

/**
 * Converts seconds to text time
 *
 * @param   int   $nSeconds   Seconds
 * @return  string   Time in text
 */
function seconds_to_time($nSeconds)
	{
	$oZeroDT = new DateTime("@0");
	$oSecondsDT = new DateTime("@$nSeconds");
	return $oZeroDT->diff($oSecondsDT)->format('%a days, %h hours, %i minutes and %s seconds');
	}

/**
 * load_config - Loads configuration PHP file into a variable
 *
 * @param   string		$sFileName			PHP file name to load
 * @param   function		$fErrorCallback	Callback function, called in case of Error
 * @return  array			Configuration array
 */
function load_config($sFileName, $fErrorCallback = null)
	{
	if (file_exists($sFileName))
		{
		$aConfigData = include $sFileName;
		return $aConfigData;
		}
	elseif (is_callable($fErrorCallback))
		{
		call_user_func($fErrorCallback);
		}
	return [];
	}

/**
 * load_view - Loads a PHP file contents as string
 *
 * @param   string	$sFileName		PHP file name to load
 * @param   array		$aVariables		Associative array of Variables used in the view, key values will be variable names.
 * @return  string	File contents
 */
function load_view($sFileName, $aVariables = [])
	{
	ob_start();
	$sContents = "";
	if (file_exists($sFileName))
		{
		extract($aVariables);
		include $sFileName;
		$sContents = ob_get_contents();
		}
	ob_end_clean();
	return $sContents;
	}

/**
 * Simple detection of html tags: text has < or >
 *
 * @return  bool	Has < or >
 */
function has_html($sText)
	{
	return strpos($sText, '<') !== false || strpos($sText, '>') !== false;
	}

/**
 * Gets MAC address
 *
 * @return  string   MAC address
 */
function get_mac()
	{
	ob_start(); // Turn on output buffering
	system('ipconfig /all'); //Execute external program to display output
	$sConsoleOutput = ob_get_contents(); // Capture the output into a variable
	ob_clean(); // Clean (erase) the output buffer
	$sMAC = substr($sConsoleOutput, (strpos($sConsoleOutput, "Physical") + 36), 17); // Get Physical Address
	return $sMAC;
	}

/**
 * Quotes INI value
 *
 * @param mixed $vValue Value
 * @return string Quoted value
 */
function quote_ini_value($vValue)
	{
	if (is_numeric($vValue))
		{
		return $vValue . "\n";
		}
	return "\"" . $vValue . "\"\n";
	}

/**
 * Writes INI File
 *
 * @param array $aSettings Settings associative array
 * @param string $sPath Path
 * @param bool $bHasSections Has Sections
 * @return  string   MAC address
 */
function write_ini_file($aSettings, $sPath, $bHasSections = FALSE)
	{
	$sContent = "";
	if ($bHasSections)
		{
		foreach ($aSettings as $nKey => $vElement)
			{
			$sContent .= "[" . $nKey . "]\n";
			foreach ($vElement as $nKey2 => $vElement2)
				{
				if (is_array($vElement2))
					{
					for ($i = 0; $i < count($vElement2); $i++)
						{
						$sContent .= $nKey2 . "[] = " . quote_ini_value($vElement2[$i]);
						}
					}
				else if ($vElement2 == "")
					$sContent .= $nKey2 . " = \n";
				else
					$sContent .= $nKey2 . " = " . quote_ini_value($vElement2);
				}
			}
		}
	else
		{
		foreach ($aSettings as $nKey => $vElement)
			{
			if (is_array($vElement))
				{
				for ($i = 0; $i < count($vElement); $i++)
					{
					$sContent .= $nKey . "[] = " . quote_ini_value($vElement[$i]);
					}
				}
			else if ($vElement == "")
				$sContent .= $nKey . " = \n";
			else
				$sContent .= $nKey . " = " . quote_ini_value($vElement);
			}
		}

	if (!$oHandle = fopen($sPath, 'w'))
		{
		return false;
		}

	$bSuccess = fwrite($oHandle, $sContent);
	fclose($oHandle);

	return $bSuccess;
	}

/**
 * sse_header - Sets Header for Data Stream 
 * 
 * @access public
 * @return void
 */
function sse_header()
	{
	header("Content-Type: text/event-stream");
	header("Cache-Control: no-cache");
	header("Connection: keep-alive");
	}

/**
 * sse_message - Sends Stream Message
 * 
 * @access public
 * @return void
 */
function sse_message($vData)
	{
	if (is_array($vData))
		{
		echo "data: " . json_encode($vData) . "\n";
		}
	else
		{
		echo "data: $vData\n";
		}
	echo "\n";
	ob_flush();
	flush();
	}

/**
 * sse_close - Sends Stream Close
 * 
 * @access public
 * @return void
 */
function sse_close()
	{
	sse_message('CLOSE');
	}
