<?php

/**
 * CSDApiSoapJobs.php - Implementation file for SellerDeck SOAP API Jobs class.
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

namespace SDExtension;

class CSDApiSoapJobs
	{

	/**
	 * @var SyncService Sync Service Object
	 */
	private $m_oSyncService;

	/**
	 * @var string $m_sApiKey API Key
	 */
	private $m_sApiKey;

	/**
	 * __construct - Object constructor method
	 * 
	 * Initializes the member variables
	 *
	 * @access public
	 * @param SyncService $oSyncService Sync Service Object
	 * @return void
	 */
	public function __construct($oSyncService = null, $sApiKey = "")
		{
		$this->m_sApiKey = $sApiKey;
		$this->m_oSyncService = new CSDApiSoapClient($oSyncService, true);
		}

	/**
	 * CommonCommand - Common part of API calls
	 * 
	 * @access public
	 * @param string $sApi API
	 * @param string $sType Type
	 * @param string $sApiKey API Key
	 * @param array $aParameters Parameters
	 * @param string $sMode Mode: single/all
	 * @return string XML
	 */
	public function CommonCommand($sApi, $sType, $aParameters = [], $sMode = '')
		{
		$aRequest = [
			'api' => $sApi,
			'apikey' => $this->m_sApiKey,
			'command' =>
			[
				'type' => $sType,
			],
		];
		if (!empty($sMode))
			{
			$aRequest['command']['mode'] = $sMode;
			}
		if (!empty($aParameters))
			{
			$aRequest['command']['parameters'] = $aParameters;
			}
		$sXML = Helper\Array2XML::createXML('request', $aRequest, ['version' => "1"])->saveXML();
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug($sXML);
		$oReturnValue = $this->m_oSyncService->InvokeApi($sXML);
		$aXML = Helper\XML2Array::createArray($oReturnValue);
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug(print_r($oReturnValue, true));
		return $aXML;
		}

	/**
	 * ConfigurationReload - Configuration Reload API call
	 * 
	 * @access public
	 * @return string XML
	 */
	public function ConfigurationReload()
		{
		return $this->CommonCommand('sdd', 'sdereloadconfig');
		}

	/**
	 * BulkStockUpdate - Bulk Stock Update
	 * 
	 * @access public
	 * @param array $aProducts Product list
	 * @return string XML
	 */
	public function BulkStockUpdate($aProducts = [])
		{
		$aParameters = [];
		foreach ($aProducts as $aProduct)
			{
			$aParameters['product'][] = [
				'mode' => 'relative',
				'prodref' => $aProduct['sProductReference'],
				'stocktoupdate' => $aProduct['nQuantityOrdered'],
			];
			if (!empty($aProduct['sPermutation']))
				{
				$aParameters['product'][] = [
					'mode' => 'relative',
					'prodref' => $aProduct['sPermutation'],
					'stocktoupdate' => $aProduct['nQuantityOrdered'],
				];
				}
			}
		return $this->CommonCommand('stock', 'update', $aParameters);
		}

	/**
	 * BulkRefreshOrders - Bulk Refresh Orders
	 * 
	 * @access public
	 * @param array $aOrders Order list
	 * @return string XML
	 */
	public function BulkRefreshOrders($aOrders = [])
		{
		$aParameters = [];
		foreach ($aOrders as $sOrderNumber)
			{
			$aParameters['order'][] = ['reference' => $sOrderNumber];
			}
		return $this->CommonCommand('order', 'display', $aParameters);
		}

	/**
	 * CommonLockAndUnlock - Common part of Lock and Unlock API calls
	 * 
	 * @access public
	 * @param string $sReference Product Reference
	 * @param array|string $vItemOrGroup Item definition
	 * @param string $sLockOrUnlock Action: 'lock' or 'unlock'
	 * @return string XML
	 */
	public function CommonLockAndUnlock($sReference, $vItemOrGroup, $sLockOrUnlock = "lock")
		{
		if (is_array($vItemOrGroup))
			{
			$aParameters = ['item' => $vItemOrGroup];
			$sReference = arr_get($vItemOrGroup, 'reference', $sReference);
			}
		else
			{
			$aParameters['item']['group'] = $vItemOrGroup;
			}
		if ("unlock" == $sLockOrUnlock)
			{
			$aParameters['item']['editaction'] = 'claimed';
			}
		$aParameters['item']['reference'] = $sReference;
		return $this->CommonCommand('sync', $sLockOrUnlock, $aParameters, 'single');
		}

	/**
	 * ProductLockAndUnlock - Common part of Lock and Unlock API calls
	 * 
	 * @access public
	 * @param string $sProductReference Product Reference
	 * @param string $sLockOrUnlock Action: 'lock' or 'unlock'
	 * @return string XML
	 */
	public function ProductLockAndUnlock($sProductReference, $sLockOrUnlock = "lock")
		{
		$aItem = [
			'group' => 'content',
			'content' => 'product',
		];
		return $this->CommonLockAndUnlock($sProductReference, $aItem, $sLockOrUnlock);
		}

	/**
	 * CommonOrderLockAndUnlock - Common part of Lock and Unlock API calls
	 * 
	 * @access private
	 * @param string $sOrderNumber Order Number
	 * @param string $sLockOrUnlock Action: 'lock' or 'unlock'
	 * @return string XML
	 */
	private function CommonOrderLockAndUnlock($sOrderNumber, $sLockOrUnlock = "lock")
		{
		return $this->CommonLockAndUnlock($sOrderNumber, 'order', $sLockOrUnlock);
		}

	/**
	 * Lock - Locking an item in EC
	 * 
	 * @access public
	 * @param string $sOrderNumber Order Number
	 * @return string XML
	 */
	public function Lock($sOrderNumber)
		{
		return $this->CommonOrderLockAndUnlock($sOrderNumber, "lock");
		}

	/**
	 * Unlock - Unlocking an item in EC
	 * 
	 * @access public
	 * @param string $sOrderNumber Order Number
	 * @return string XML
	 */
	public function Unlock($sOrderNumber)
		{
		return $this->CommonOrderLockAndUnlock($sOrderNumber, "unlock");
		}

	}
