<?php

/**
 * CAbstractHandshaking.php - Implementation file for Handshaking Abstract class.
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */
abstract class CAbstractHandshaking
	{

	/**
	 * @var array $m_aInitMessage Init Message
	 */
	protected $m_aInitMessage = [];

	/**
	 * __construct - Object constructor
	 *
	 * @access public
	 * @param array $aInitMessage Init Message
	 * @return void
	 */
	public function __construct($aInitMessage)
		{
		if (empty($aInitMessage) || false === $aInitMessage)
			{
			define('HANDSHAKING', true);
			require_once(FRAMEWORK_ROOT . 'bootstrap.php');
			\SDExtension\Helper\CLogger::get('unknown', CACHE_ROOT)->addError(EXCEPTION_MSG_NO_DATA);
			throw new \SDExtension\SDException\SDException(EXCEPTION_MSG_NO_DATA, EXCEPTION_CODE_NO_DATA);
			}
		$this->m_aInitMessage = $aInitMessage;
		return $this;
		}

	/**
	 * GetResponse - Getting Response
	 *
	 * @access public
	 * @param array $aInitMessage Init Message
	 * @return string Response XML
	 */
	public function GetResponse()
		{
		$this->SetHeader();
		switch ($this->GetFunction())
			{
			case HANDSHAKE_VERSION_INFORMATION:
				define('HANDSHAKING', true);
				require_once(FRAMEWORK_ROOT . 'bootstrap.php');
				return $this->VersionInformation();
				break;
			case HANDSHAKE_SDD_CONFIGURATION:
				define('HANDSHAKING', true);
				require_once(FRAMEWORK_ROOT . 'bootstrap.php');
				return $this->SDDConfiguration();
				break;
			case HANDSHAKE_EXTENSION_CONFIGURATION:
				require_once(FRAMEWORK_ROOT . 'bootstrap.php');
				return $this->ExtensionConfiguration();
				break;
			default:
			}
		}

	/**
	 * SetHeader - Setting Response XML header
	 *
	 * @access public
	 * @param string $sHeader Response XML header
	 * @return CAbstractHandshaking This object
	 */
	public function SetHeader($sHeader = "Content-Type: application/xml; charset=utf-8")
		{
//
// Response XML header
//
		header($sHeader);
		return $this;
		}

	/**
	 * GetFunction - Getting Function name
	 *
	 * @access private
	 * @return string Function name
	 */
	private function GetFunction()
		{
		return @arr_get($this->m_aInitMessage, '@attributes.name', '');
		}

	/**
	 * VersionInformation - Version Information
	 * 
	 * Handshaking round #1
	 * 
	 * @access protected
	 * @return string Response XML
	 */
	protected function VersionInformation()
		{
		$sConfigFileVersion = CACHE_ROOT . "version.php";
		@save_config($this->m_aInitMessage, true, $sConfigFileVersion);
		$nActualVersionNumber = @arr_get($this->m_aInitMessage, 'versionid.major', 1) * 10000 + @arr_get($this->m_aInitMessage, 'versionid.minor', 0) * 100 + @arr_get($this->m_aInitMessage, 'versionid.maintenance', 0);
		$sActualVersion = @arr_get($this->m_aInitMessage, 'versionid.major', 1) . "." . @arr_get($this->m_aInitMessage, 'versionid.minor', 0) . "." . @arr_get($this->m_aInitMessage, 'versionid.maintenance', 0);
		$nRequiredVersionNumber = REQUIRED_SDD_VERSION_MAJOR * 10000 + REQUIRED_SDD_VERSION_MINOR * 100 + REQUIRED_SDD_VERSION_MAINTENANCE;
		if ($nActualVersionNumber < $nRequiredVersionNumber)
			{
			$nRequiredVersion = REQUIRED_SDD_VERSION_MAJOR . "." . REQUIRED_SDD_VERSION_MINOR . "." . REQUIRED_SDD_VERSION_MAINTENANCE;
			\SDExtension\Helper\CLogger::get('unknown', CACHE_ROOT)->addError(sprintf(EXCEPTION_MSG_SDD_INCOMPATIBLE, $nRequiredVersion));
			}
		$sConfigFile = CONFIG_APP_ROOT . "handshake_response1.php";
		$aResponse = load_config($sConfigFile, function() use ($sConfigFile)
			{
			\SDExtension\Helper\CLogger::get('unknown', CACHE_ROOT)->addError(EXCEPTION_MSG_CANT_LOAD_CONFIG_FILE . ": $sConfigFile.");
			throw new \SDExtension\SDException\SDException(EXCEPTION_MSG_CANT_LOAD_CONFIG_FILE . ": $sConfigFile.", EXCEPTION_CODE_CANT_LOAD_CONFIG_FILE);
			});
		$sResponseXML = SDExtension\Helper\Array2XML::createXML('response', $aResponse)->saveXML();
		return $sResponseXML;
		}

	/**
	 * SDDConfiguration - SellerDeck Desktop Configuration
	 * 
	 * Handshaking round #2
	 * 
	 * @access protected
	 * @return string Response XML
	 */
	protected function SDDConfiguration()
		{
		$sSiteName = arr_get($this->m_aInitMessage, 'site.sitename.sitename', DEFAULT_SITE_NAME);
		$sSdeLogs = arr_get($this->m_aInitMessage, 'paths.logs.site', DEFAULT_LOG_ROOT);
		$sSiteSdeDataFolder = arr_get($this->m_aInitMessage, 'paths.files.temporary', DEFAULT_CONFIG_ROOT) . DS;
		if (!file_exists($sSdeLogs))
			{
			@mkdir($sSdeLogs, 0777, true);
			}
		if (!file_exists($sSiteSdeDataFolder))
			{
			@mkdir($sSiteSdeDataFolder, 0777, true);
			}
		@rename(CACHE_ROOT . "version.php", $sSiteSdeDataFolder . "version.php");
		@save_config($this->m_aInitMessage, false, $sSiteSdeDataFolder . "details.php");
		@save_config([
				'sSiteName' => $sSiteName,
				'sSdeLogs' => $sSdeLogs,
				'sSiteSdeDataFolder' => $sSiteSdeDataFolder,
				], false, CACHE_ROOT . "details_temp.php");
		$sConfigFile = CONFIG_APP_ROOT . "handshake_response2.php";
		$aResponse = load_config($sConfigFile, function() use ($sConfigFile)
			{
			\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addError(EXCEPTION_MSG_CANT_LOAD_CONFIG_FILE . ": $sConfigFile.");
			throw new \SDExtension\SDException\SDException(EXCEPTION_MSG_CANT_LOAD_CONFIG_FILE . ": $sConfigFile.", EXCEPTION_CODE_CANT_LOAD_CONFIG_FILE);
			});
		$sResponseXML = SDExtension\Helper\Array2XML::createXML('response', $aResponse)->saveXML();
		@SDExtension\Helper\CLogger::init($sSiteName, $sSdeLogs . DS);
		@SDExtension\Helper\CLogger::get()->addInfo("Handshaking #2");
		return $sResponseXML;
		}

	/**
	 * ExtensionConfiguration - Extension Configuration
	 * 
	 * Handshaking round #3
	 * 
	 * @access protected
	 * @return string Response XML
	 */
	protected function ExtensionConfiguration()
		{
		//
		// Init Settings
		//
		$this->AppInit();
		$sConfigFile = CONFIG_APP_ROOT . "handshake_response3.php";
		$aResponse = load_config($sConfigFile, function() use ($sConfigFile)
			{
			\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addError(EXCEPTION_MSG_CANT_LOAD_CONFIG_FILE . ": $sConfigFile.");
			throw new \SDExtension\SDException\SDException(EXCEPTION_MSG_CANT_LOAD_CONFIG_FILE . ": $sConfigFile.", EXCEPTION_CODE_CANT_LOAD_CONFIG_FILE);
			});
		$sResponseXML = SDExtension\Helper\Array2XML::createXML('response', $aResponse)->saveXML();
		SDExtension\Helper\CLogger::get()->addInfo("Handshaking done!");
		//
		// Unlocking
		//
		@RemoveLock();
		return $sResponseXML;
		}

	/**
	 * AppInit - Init Application
	 * 
	 * @access protected
	 * @return void
	 */
	abstract protected function AppInit();
	}
