<?php

/**
 * CDataBase.php - Implementation file for DataBase class.
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erd?di
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

namespace SDExtension\DB;

class CDataBase
	{

	/**
	 * @var object DataBase Object
	 */
	protected $m_oDB = null;

	/**
	 * @var string DataBase ID
	 */
	protected $m_sDataBaseID = DB_CATALOG;

	/**
	 * @var string DataBase Authentication Type
	 */
	protected $m_sDBAuthType = "DB";

	/**
	 * @var string DataBase User
	 */
	protected $m_sDBUser = "dbo";

	/**
	 * @var object DataBases
	 */
	protected $m_aDataBases = [];

	/**
	 * @var string Begin quote sign for DB Columns
	 */
	protected $m_sColumnQuoteBegin = '`';

	/**
	 * @var string End quote sign for DB Columns
	 */
	protected $m_sColumnQuoteEnd = '`';

	/**
	 * @var string Begin quote sign for Values
	 */
	protected $m_sValueQuoteBegin = "'";

	/**
	 * @var string End quote sign for Values
	 */
	protected $m_sValueQuoteEnd = "'";

	/**
	 * @var string Null Value
	 */
	protected $m_sNullValue = "NULL";

	/**
	 * @var string Last Query
	 */
	protected $m_sLastQuery = "";

	/**
	 * __construct - Object constructor method
	 *
	 * @access public
	 * @param array $aConnectionDetails
	 * @return void
	 */
	public function __construct($aConnectionDetails, $sDatabaseID)
		{
		$sDsn = arr_get($aConnectionDetails, 'sDsn', '');
		$this->m_sDBUser = arr_get($aConnectionDetails, 'sUser', '');
		$sPassword = arr_get($aConnectionDetails, 'sPassword', '');
		$this->m_sDataBaseID = arr_get($aConnectionDetails, 'sDatabaseID', '');
		$this->m_sDBAuthType = arr_get($aConnectionDetails, 'sDBAuthType', '');
		$this->m_aDataBases = arr_get($aConnectionDetails, 'aDataBases', []);
		$this->m_oDB = new \PDO($sDsn, $this->m_sDBUser, $sPassword);
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addInfo("Connected to Database: $sDatabaseID");
		}

	/**
	 * GetDatabaseID - Database ID
	 *
	 * @access public
	 * @return string Database ID
	 */
	public function GetDatabaseID()
		{
		return $this->m_sDataBaseID;
		}

	/**
	 * GetDataBases - DataBases array
	 *
	 * @access public
	 * @return array DataBase
	 */
	public function GetDataBases()
		{
		return $this->m_aDataBases;
		}

	/**
	 * GetDBUser - DataBase User
	 *
	 * @access public
	 * @return string DataBase User
	 */
	public function GetDBUser()
		{
		return $this->m_sDBUser;
		}

	/**
	 * GetDBAuthType - DataBase Authentication Type
	 *
	 * @access public
	 * @return string DataBase Authentication Type
	 */
	public function GetDBAuthType()
		{
		return $this->m_sDBAuthType;
		}

	/**
	 * Expression - Mark SQL Expression to act as CDataBase Expression (ie. not to be surounded by quotes)
	 *
	 * @access public
	 * @param string $sExpression SQL Expression to be marked as CDataBase expression 
	 * @return string Marked as expression
	 */
	public function Expression($sExpression)
		{
		return SQL_EXPRESSION_PREFIX . $sExpression;
		}

	/**
	 * Quote - Surrounds string with given prefix and suffix (quote signs)
	 *
	 * @access protected
	 * @param string $sData Data to quote
	 * @param string $sQuoteBegin Begin quote sign
	 * @param string $sQuoteBegin End quote sign
	 * @return string Quoted string
	 */
	protected function Quote($sData, $sQuoteBegin, $sQuoteEnd)
		{
		return $sQuoteBegin . $sData . $sQuoteEnd;
		}

	/**
	 * Sanitize - Sanitizes string
	 *
	 * @access protected
	 * @param string $sData Data to sanitize
	 * @return string Sanitized string
	 */
	protected function Sanitize($sData)
		{
		return addslashes($sData);
		}

	/**
	 * QuoteValue - Quotes Value string
	 *
	 * @access public
	 * @param mixed $vData Data to quote
	 * @return string Quoted string
	 */
	public function QuoteValue($vData)
		{
		if (starts_with($vData, SQL_COLUMN))
			{
			$vData = remove_prefix($vData, SQL_COLUMN);
			return $this->Quote($vData, $this->m_sColumnQuoteBegin, $this->m_sColumnQuoteEnd);
			}
		if (starts_with($vData, SQL_TYPE_STRING))
			{
			$vData = remove_prefix($vData, SQL_TYPE_STRING);
			$vData = $this->Sanitize($vData);
			return $this->Quote($vData, $this->m_sValueQuoteBegin, $this->m_sValueQuoteEnd);
			}
		if (starts_with($vData, SQL_EXPRESSION_PREFIX))
			{
			return remove_prefix($vData, SQL_EXPRESSION_PREFIX);
			}
		if (is_bool($vData))
			{
			return $this->BoolValue($vData);
			}
		if (is_numeric($vData))
			{
			return $vData;
			}
		if ($vData == null)
			{
			return $this->m_sNullValue;
			}
		$vData = $this->Sanitize($vData);
		return $this->Quote($vData, $this->m_sValueQuoteBegin, $this->m_sValueQuoteEnd);
		}

	/**
	 * QuoteTable - Quotes Table name
	 *
	 * @access public
	 * @param string|array $vTableName Table name
	 * @return string Quoted Table name
	 */
	public function QuoteTable($vTableName)
		{
		if (is_array($vTableName))
			{
			$sTableName = array_shift($vTableName);
			$sTableAlias = array_shift($vTableName);
			return
					$this->Quote($sTableName, $this->m_sColumnQuoteBegin, $this->m_sColumnQuoteEnd) .
					(!empty($sTableAlias) ? " AS " .
							$this->Quote($sTableAlias, $this->m_sColumnQuoteBegin, $this->m_sColumnQuoteEnd) : "");
			}
		else
			{
			return $this->Quote($vTableName, $this->m_sColumnQuoteBegin, $this->m_sColumnQuoteEnd);
			}
		}

	/**
	 * QuoteColumn - Quotes DB Column string
	 *
	 * @access public
	 * @param string|array $vData Data to quote
	 * @return string Quoted string
	 */
	public function QuoteColumn($vData)
		{
		$sTableName = "";
		if (is_array($vData))
			{
			$sTableName = array_shift($vData);
			$vData = array_shift($vData);
			if (empty($vData))
				{
				$vData = $sTableName;
				}
			else
				{
				$sTableName = $this->Quote($sTableName, $this->m_sColumnQuoteBegin, $this->m_sColumnQuoteEnd) . ".";
				}
			}
		if (starts_with($vData, SQL_EXPRESSION_PREFIX))
			{
			return remove_prefix($vData, SQL_EXPRESSION_PREFIX);
			}
		if (starts_with($vData, SQL_DATA_PREFIX))
			{
			return $this->QuoteValue(remove_prefix($vData, SQL_DATA_PREFIX));
			}
		if ($vData == "*")
			{
			return $vData;
			}
		return $sTableName . $this->Quote($vData, $this->m_sColumnQuoteBegin, $this->m_sColumnQuoteEnd);
		}

	/**
	 * QuoteCompundColumn - Quotes DB Column string or array
	 *
	 * @access public
	 * @param string|array $vUnquotedElement Data to quote
	 * @return string Quoted string
	 */
	public function QuoteCompundColumn($vUnquotedElement)
		{
		if (is_array($vUnquotedElement))
			{
			$sAlias = "";
			if (count($vUnquotedElement) > 2)
				{
				$sAlias = " AS " . $this->QuoteColumn(array_pop($vUnquotedElement));
				}
			return implode('.', array_map(array($this, 'QuoteColumn'), $vUnquotedElement)) . $sAlias;
			}
		else
			{
			return $this->QuoteColumn($vUnquotedElement);
			}
		}

	/**
	 * QuoteArray - Quotes array elements
	 *
	 * @access protected
	 * @param array $aUnquotedArray Array to quote
	 * @param bool $bValue 
	 * @return array Quoted array
	 */
	protected function QuoteArray($aUnquotedArray, $bValue = true)
		{
		$aQuotedArray = [];
		foreach ($aUnquotedArray as $sData)
			{
			if ($bValue)
				{
				$aQuotedArray[] = $this->QuoteValue($sData);
				}
			else
				{
				$aQuotedArray[] = $this->QuoteColumn($sData);
				}
			}
		return $aQuotedArray;
		}

	/**
	 * QuoteUpdateArray - Generates array of quoted parameters used in UPDATE expressions
	 *
	 * @access protected
	 * @param array $aUnquotedArray Array to quote
	 * @return string Quoted string
	 */
	protected function QuoteUpdateArray($aUnquotedArray)
		{
		$aQuotedArray = array();
		foreach ($aUnquotedArray as $sKey => $sData)
			{
			$aQuotedArray[] = $this->QuoteColumn($sKey) .
					" = " .
					$this->QuoteValue($sData);
			}
		return implode(", ", $aQuotedArray);
		}

	/**
	 * QuoteValueArray - Quotes array elements, with Value quotes
	 *
	 * @access protected
	 * @param array $aUnquotedArray Array to quote
	 * @return array Quoted array
	 */
	protected function QuoteValueArray($aUnquotedArray)
		{
		return $this->QuoteArray($aUnquotedArray, true);
		}

	/**
	 * QuoteColumnArray - Quotes array elements, with DB Column quotes
	 *
	 * @access protected
	 * @param array $aUnquotedArray Array to quote
	 * @return array Quoted array
	 */
	protected function QuoteColumnArray($aUnquotedArray)
		{
		return $this->QuoteArray($aUnquotedArray, false);
		}

	/**
	 * QuoteValueList - Generates string of quoted Values used in INSERT expressions
	 *
	 * @access protected
	 * @param array $aUnquotedArray Array to quote
	 * @param string $sSeparator Separator
	 * @return array Quoted array
	 */
	protected function QuoteValueList($aUnquotedArray, $sSeparator = ", ")
		{
		$sList = implode($sSeparator, $this->QuoteValueArray($aUnquotedArray));
		return $sList;
		}

	/**
	 * QuoteColumnList - Generates string of quoted DB Columns used in INSERT expressions
	 *
	 * @access protected
	 * @param array $aUnquotedArray Array to quote
	 * @param string $sSeparator Separator
	 * @return array Quoted array
	 */
	protected function QuoteColumnList($aUnquotedArray, $sSeparator = ", ")
		{
		$aProcessedUnquotedArray = array();
		foreach ($aUnquotedArray as $vUnquotedElement)
			{
			$aProcessedUnquotedArray[] = $this->QuoteCompundColumn($vUnquotedElement);
			}
		$sList = implode($sSeparator, $aProcessedUnquotedArray);
		return $sList;
		}

	/**
	 * GetDB - DB
	 *
	 * @access public
	 * @return object DB
	 */
	public function GetDB()
		{
		return $this->m_oDB;
		}

	/**
	 * GetLastQuery - Returns last query SQL
	 *
	 * @access public
	 * @return string Quoted array
	 */
	public function GetLastQuery()
		{
		return $this->m_sLastQuery;
		}

	/**
	 * SQL query
	 *
	 * @access public
	 * @param string $sQuery Table name
	 * @param int $nReturnType Return Type (associative array by default)
	 * @return array|object Query result
	 */
	public function SQL($sQuery, $nReturnType = \PDO::FETCH_ASSOC)
		{
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug(__FUNCTION__ . " : " . $sQuery);
		return $this->m_oDB->query($sQuery, $nReturnType);
		}

	/**
	 * BasicInsert - Generates and runs Basic INSERT expression
	 *
	 * @access protected
	 * @param string $sTable Table name
	 * @param array $aValues Array of (DB Column => Value) pairs
	 * @return null
	 */
	protected function BasicInsert($sTable, $aValues)
		{
		$sTableQuoted = $this->QuoteTable($sTable);
		$sColumnList = $this->QuoteColumnList(array_keys($aValues));
		$sValueList = $this->QuoteValueList(array_values($aValues));
		$sQuery = "INSERT INTO $sTableQuoted (" . $sColumnList . ") VALUES (" . $sValueList . ");";
		$nResult = $this->m_oDB->exec($sQuery);
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug(__FUNCTION__ . " : $sQuery");
		return null;
		}

	/**
	 * Insert - Generates and runs INSERT expression
	 *
	 * @todo "OR" is missing from WHERE
	 * 
	 * @access public
	 * @param string $sTable Table name
	 * @param array $aValues Array of (DB Column => Value) pairs
	 * @param bool $bAutoID Has Auto ID?
	 * @param string $sIDColumn ID Column
	 * @return null | mixed Last inserted ID
	 */
	public function Insert($sTable, $aValues, $bAutoID = true, $sIDColumn = null)
		{
		//
		// Default INSERT, without ID column
		//
	if ($sIDColumn == null || $bAutoID)
			{
			return $this->BasicInsert($sTable, $aValues);
			}
		$sQuery = "";
		$sTableQuoted = $this->QuoteTable($sTable);
		$bInserted = false;
		$nCounter = 0;
		$nLastInsertID = 0;
		//
		// Repeated INSERT
		//
	while (!$bInserted && $nCounter < MAX_RETRY_BY_INSERT)
			{
			$sIDColumnQuoted = $this->QuoteColumn($sIDColumn);
			$sMaxID = "MaxIDOf$sTable";
			$oMaxIDResult = $this->Select($sTable, [SQL_EXPRESSION_PREFIX . "MAX($sIDColumnQuoted) AS $sMaxID"]);
			//
			// Can't read Table
			//
	   if ($oMaxIDResult === false)
				{
				\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addWarning("Can't read $sTable when getting Last ID!");
				return null;
				}
			$aMaxIDResult = $oMaxIDResult->fetch(\PDO::FETCH_ASSOC);
			$nMaxID = $aMaxIDResult[$sMaxID];
			//
			// First record
			//
	   if (empty($nMaxID))
				{
				$nMaxID = 1;
				}
			//
			// Not numeric value: can't be increased!
			//
	   elseif (!is_numeric($nMaxID))
				{
				\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addWarning("Table ID [$sTable.$sIDColumn] is not numeric [$nMaxID]!");
				return null;
				}
			//
			// Attepmting INSERT
			//
			$nNewID = $nMaxID + INCREASE_ID_BY_INSERT;
			$aValues[$sIDColumn] = $nNewID;
			$sColumnList = $this->QuoteColumnList(array_keys($aValues));
			$sValueList = $this->QuoteValueList(array_values($aValues));
			$sQuery = "INSERT INTO $sTableQuoted (" . $sColumnList . ") VALUES (" . $sValueList . ");";
			$nResult = $this->m_oDB->exec($sQuery);
			//
			// Unsuccessful #1 invalid query?
			//
	   if ($nResult === false)
				{
				\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addWarning("Can't INSERT into table [$sTable] with ID [$sIDColumn] value $nNewID!");
				}
			//
			// Unsuccessful #2 zero records inserted.
			//
	   elseif ($nResult <= 0)
				{
				\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addWarning("0 records inserted.");
				}
			//
			// Insertion successful
			//
	   else
				{
				$nLastInsertID = $nNewID;
				$bInserted = true;
				}
			$nCounter++;
			}
		$this->m_sLastQuery = $sQuery;
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug(__FUNCTION__ . " : " . $sQuery);
		//
		// Final insertion unsuccessful
		//
	if (!$bInserted)
			{
			\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addWarning("Can't INSERT into table [$sTable] with ID [$sIDColumn] value $nNewID!");
			return null;
			}
		return $nLastInsertID;
		}

	/**
	 * Update - Generates and runs UPDATE expression
	 *
	 * @todo "OR" is missing from WHERE
	 * 
	 * @access public
	 * @param string $sTable Table name
	 * @param array $aValues Array of (DB Column => Value) pairs
	 * @param array $aWhere Array of conditions
	 * @return int Number of rows that were modified or deleted 
	 */
	public function Update($sTable, $aValues, $aWhere = [[1, '', '']])
		{
		$sSetList = $this->QuoteUpdateArray($aValues);
		$sTable = $this->QuoteTable($sTable);
		$sQuery = "UPDATE $sTable SET " . $sSetList . " WHERE " . $this->CompileWhere($aWhere) . ";";
		$this->m_sLastQuery = $sQuery;
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug(__FUNCTION__ . " : " . $sQuery);
		return $this->m_oDB->exec($sQuery);
		}

	/**
	 * Select - Generates and runs SELECT expression
	 *
	 * @todo "OR" is missing from WHERE
	 * 
	 * @access public
	 * @param string $sTable Table name
	 * @param string|array $vColumns Columns to select
	 * @param array $aWhere Array of conditions
	 * @param array $aOthers Other parameters
	 * @param int $nReturnType Return Type (associative array by default)
	 * @return array|object Query result
	 */
	public function Select($sTable, $vColumns = "*", $aWhere = [], $aOthers = [], $nReturnType = \PDO::FETCH_ASSOC)
		{
		$aQuery = [];
		if (is_array($vColumns))
			{
			$aQuery["select"] = $this->QuoteColumnList($vColumns);
			}
		else
			{
			$aQuery["select"] = $vColumns;
			}
		$aQuery["from"] = $this->QuoteTable($sTable);
		if (!empty($aWhere))
			{
			//
			// TODO: "OR" is missing...
			//
            $aQuery["where"] = $this->CompileWhere($aWhere);
			}
		$aQuery["join"] = $this->CompileJoin(arr_get($aOthers, "join", []));
		$aQuery["group_by"] = $this->CompileGroupBy(arr_get($aOthers, "group_by", []));
		$aQuery["order_by"] = $this->CompileOrderBy(arr_get($aOthers, "order_by", []));
		$aQuery["limit"] = $this->CompileLimit(arr_get($aOthers, "limit", []));
		$sQuery = $this->CompileSelectSql($aQuery);
		$this->m_sLastQuery = $sQuery;
		\SDExtension\Helper\CLogger::get(LOG_CHANNEL_NAME)->addDebug(__FUNCTION__ . " : " . $sQuery);
		return $this->m_oDB->query($sQuery, $nReturnType);
		}

	/**
	 * CompileSelectSql - Compiles SQL expression
	 * 
	 * @access protected
	 * @param array $aWhere Array of conditions
	 * @return string WHERE statement
	 */
	protected function CompileSelectSql($aQuery = [])
		{
		$sSelectSQL = "";
		if (!empty($aQuery["select"]))
			{
			$sSelectSQL .= "SELECT $aQuery[select] ";
			}
		else
			{
			$sSelectSQL .= "SELECT * ";
			}
		if (!empty($aQuery["from"]))
			{
			$sSelectSQL .= "FROM $aQuery[from] ";
			}
		if (!empty($aQuery["join"]))
			{
			$sSelectSQL .= " $aQuery[join] ";
			}
		if (!empty($aQuery["where"]))
			{
			$sSelectSQL .= "WHERE $aQuery[where] ";
			}
		if (!empty($aQuery["group_by"]))
			{
			$sSelectSQL .= "GROUP BY $aQuery[group_by] ";
			}
		if (!empty($aQuery["order_by"]))
			{
			$sSelectSQL .= "ORDER BY $aQuery[order_by] ";
			}
		if (!empty($aQuery["limit"]))
			{
			$sSelectSQL .= "LIMIT $aQuery[limit] ";
			}
		return $sSelectSQL;
		}

	/**
	 * CompileWhereElement - Compiles a single WHERE element
	 * 
	 * @access protected
	 * @param array $aValue Array of a single condition
	 * @return string WHERE statement
	 */
	protected function CompileWhereElement($aValue = [])
		{
		if (count($aValue) == 3)
			{
			return $this->QuoteColumn($aValue[0]) .
					" $aValue[1] " .
					$this->QuoteValue($aValue[2]);
			}
		elseif (count($aValue) == 2)
			{
			$sWhere = $this->QuoteColumn($aValue[0]);
			if (starts_with($aValue[1], SQL_EXPRESSION_PREFIX))
				{
				return $sWhere .
						" " .
						remove_prefix($aValue[1], SQL_EXPRESSION_PREFIX);
				}
			elseif (starts_with($aValue[1], SQL_TYPE_STRING))
				{
				return $sWhere .
						" LIKE " .
						$this->QuoteValue($aValue[1]);
				}
			else
				{
				return $sWhere .
						" = " .
						$this->QuoteValue($aValue[1]);
				}
			}
		else
			{
			return $aValue[0];
			}
		}

	/**
	 * IsElementFormat - Checks if value is a well formatted WHERE element 
	 * 
	 * @access protected
	 * @param array $aValue Value
	 * @return bool
	 */
	protected function IsElementFormat($aValue)
		{
		if (is_array($aValue) && isset($aValue[0]))
			{
			if (is_array($aValue[0]) && isset($aValue[0][0]))
				{
				return !is_array($aValue[0][0]);
				}
			return true;
			}
		return false;
		}

	/**
	 * CompileWhere - Generates WHERE expression
	 * 
	 * Used in SELECT and UPDATE
	 *
	 * @todo "OR" is missing from WHERE
	 * 
	 * @access protected
	 * @param array $aWhere Array of conditions
	 * @return string WHERE statement
	 */
	public function CompileWhere($aWhere = [], $aBoolOp = BOOP_OP_AND)
		{
		$aWhereList = [];
		if (is_array($aWhere))
			{
			foreach ($aWhere as $vKey => $aValue)
				{
				if (BOOP_OP_AND === $vKey || BOOP_OP_OR === $vKey)
					{
					$aWhereList[] = $this->CompileWhere($aValue, $vKey);
					}
				else
					{
					if ($this->IsElementFormat($aValue))
						{
						$aWhereList[] = $this->CompileWhereElement($aValue);
						}
					else
						{
						$aWhereList[] = "(" . $this->CompileWhere($aValue, $aBoolOp) . ")";
						}
					}
				}
			}
		return implode(" $aBoolOp ", $aWhereList);
		}

	/**
	 * CompileJoinOn - Generates JOIN ... ON expression
	 * 
	 * @todo "OR" is missing from WHERE
	 * 
	 * @access protected
	 * @param array $aJoinOn Array of conditions
	 * @return string WHERE statement
	 */
	public function CompileJoinOn($aJoinOn = [])
		{
		$aJoinOnList = [];
		foreach ($aJoinOn as $sKey => $aValue)
			{
			if (is_array($aValue))
				{
				if (count($aValue) == 3)
					{
					$aJoinOnList[] = $this->QuoteCompundColumn($aValue[0]) .
							" $aValue[1] " .
							$this->QuoteCompundColumn($aValue[2]);
					}
				elseif (count($aValue) == 2)
					{
					$aJoinOnList[] = $this->QuoteCompundColumn($aValue[0]) .
							" = " .
							$this->QuoteCompundColumn($aValue[1]);
					}
				}
			}
		return implode(' AND ', $aJoinOnList);
		}

	/**
	 * CompileLimit - Compiles LIMIT expression
	 * 
	 * @access protected
	 * @param array|string $vLimit Array or String of LIMIT(s)
	 * @return string LIMIT expression
	 */
	protected function CompileLimit($vLimit)
		{
		if (is_array($vLimit))
			{
			return implode(", ", $vLimit);
			}
		else
			{
			return $vLimit;
			}
		}

	/**
	 * CompileJoin - Compiles JOIN expression
	 * 
	 * @access protected
	 * @param array $aJoins Array of JOINs
	 * @return string JOIN expression
	 */
	protected function CompileJoin($aJoins)
		{
		$aJoinDirections = [];
		$aJoinTables = [];
		$aJoinOns = [];
		foreach ($aJoins as $aJoin)
			{
			$sJoinTable = "";
			switch (strtolower(arr_get($aJoin, 0, "inner")))
				{
				case "right":
					$aJoinDirections[] = "RIGHT JOIN";
					break;
				case "left":
					$aJoinDirections[] = "LEFT JOIN";
					break;
				default:
					$aJoinDirections[] = "INNER JOIN";
					break;
				}
			$aJoinTables[] = $this->QuoteTable(arr_get($aJoin, 1, ""));
			$aJoinOns[] = arr_get($aJoin, 2, []);
			}
		return $this->ConcatenateJoins($aJoinDirections, $aJoinTables, $aJoinOns);
		}

	/**
	 * ConcatenateJoins - Concatenates JOIN expression
	 * 
	 * @access protected
	 * @param array $aJoinDirections Array of JOIN Directions
	 * @param array $aJoinTables Array of JOIN Tables
	 * @param array $aJoinOns Array of JOIN ONs
	 * @return string JOIN expression
	 */
	protected function ConcatenateJoins($aJoinDirections, $aJoinTables, $aJoinOns)
		{
		$sJoins = "";
		foreach ($aJoinTables as $nKey => $sJoinTable)
			{
			$sJoins .= arr_get($aJoinDirections, $nKey, "INNER JOIN") . " $sJoinTable ON (" . $this->CompileJoinOn(arr_get($aJoinOns, $nKey, "")) . ") ";
			}
		return $sJoins;
		}

	/**
	 * CompileGroupBy - Compiles GROUP BY expression
	 * 
	 * @access protected
	 * @param array $aGroupByList Array of GROUP BYs
	 * @return string GROUP BY expression
	 */
	protected function CompileGroupBy($aGroupByList)
		{
		$aGroupBy = [];
		foreach ($aGroupByList as $nKey => $aGroupByEntry)
			{
			$aGroupBy[] = $this->QuoteCompundColumn(arr_get($aGroupByEntry, 0, "")) . "." .
					$this->QuoteCompundColumn(arr_get($aGroupByEntry, 1, ""));
			}
		return implode(", ", $aGroupBy);
		}

	/**
	 * CompileOrderBy - Compiles ORDER BY expression
	 * 
	 * @access protected
	 * @param array $aOrderByList Array of ORDER BYs
	 * @return string ORDER BY expression
	 */
	protected function CompileOrderBy($aOrderByList)
		{
		$aOrderBy = [];
		foreach ($aOrderByList as $nKey => $aOrderByEntry)
			{
			$aOrderBy[] = $this->QuoteCompundColumn(arr_get($aOrderByEntry, 0, ""))
					. " "
					. strtoupper(arr_get($aOrderByEntry, 1, "ASC"));
			}
		return implode(", ", $aOrderBy);
		}

	/**
	 * BoolValue - Converts Boolean to appropriate value depending on driver
	 * 
	 * @access protected
	 * @param bool $bData Boolean value
	 * @return bool
	 */
	protected function BoolValue($bData)
		{
		return $bData;
		}

	}
