<?php

/**
 * CDataBaseFactory.php - Implementation file for DataBase Factory class.
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

namespace SDExtension\DB;

class CDataBaseFactory
	{

	/**
	 * @var object $m_oCatalogDB Catalog DB
	 */
	static $m_oCatalogDB = null;

	/**
	 * @var object $m_oShippingDB Shipping DB
	 */
	static $m_oShippingDB = null;

	/**
	 * @var bool $m_bPDO PDO
	 */
	static $m_bPDO = true;

	/**
	 * createAll - Creates all Database connections
	 *
	 * @access public
	 * @param string $aConfiguration Database Configuration
	 * @return array Array of CDataBase objects
	 */
	public static function createAll($aConfiguration)
		{
		return [
			static::GetCatalogDB($aConfiguration),
			static::GetShippingDB($aConfiguration),
		];
		}

	/**
	 * GetCatalogDB - Gets or Creates Catalog Database connection
	 *
	 * @access public
	 * @param array $aConfiguration Database Configuration
	 * @return CDataBase
	 */
	public static function GetCatalogDB($aConfiguration = [])
		{
		if (null == static::$m_oCatalogDB)
			{
			static::$m_oCatalogDB = static::CreateConnection($aConfiguration, DB_CATALOG);
			}
		return static::$m_oCatalogDB;
		}

	/**
	 * GetShippingDB - Gets or Creates Shipping Database connection
	 *
	 * @access public
	 * @param array $aConfiguration Database Configuration
	 * @return CDataBase
	 */
	public static function GetShippingDB($aConfiguration = [])
		{
		if (null == static::$m_oShippingDB)
			{
			static::$m_oShippingDB = static::CreateConnection($aConfiguration, DB_SHIPPING);
			}
		return static::$m_oShippingDB;
		}

	/**
	 * CreateConnection - Creates Database connection
	 *
	 * @access protected
	 * @param array $aConfiguration Connection Details
	 * @param string $sDatabaseID Database ID
	 * @return CDataBase DataBase object
	 */
	protected static function CreateConnection($aConfiguration, $sDatabaseID = DB_CATALOG)
		{
		static::$m_bPDO = is_PDO();
		$sDBType = arr_get($aConfiguration, 'dbtype', DB_TYPE_ACCESS);
		$sDBAuthType = "";
		switch ($sDBType)
			{
			case DB_TYPE_ACCESS:
				$sDsn = static::createAccessDsn($aConfiguration, $sDatabaseID);
				$sDriverClass = static::$m_bPDO ? DB_CLASS_ACCESS : DB_CLASS_ACCESS_NO_PDO;
				$sUser = "";
				$sPassword = "";
				$aDataBases = [
					DB_CATALOG => arr_get($aConfiguration, 'catalogdbfilepath', 'Site1\\ActinicCatalog.mdb'),
					DB_SHIPPING => arr_get($aConfiguration, 'shippingdbfilepath', 'Site1\\ShipControl\\SimpleShipping.mdb'),
				];
				break;
			case DB_TYPE_SQLSERVER:
				$sDsn = static::createSQLServerDsn($aConfiguration, $sDatabaseID);
				$sDriverClass = DB_CLASS_SQLSERVER;
				$sUser = arr_get($aConfiguration, 'username', '');
				$sPassword = arr_get($aConfiguration, 'userpwd', '');
				$sDBAuthType = arr_get($aConfiguration, 'dbauthtype', '');
				$aDataBases = [
					DB_CATALOG => arr_get($aConfiguration, 'catalogdbsqlname', 'NewSite_Catalog'),
					DB_SHIPPING => arr_get($aConfiguration, 'shippingdbsqlname', 'NewSite_Shipping'),
				];
				break;
			}
		$aConnectionDetails = [
			'sDsn' => $sDsn,
			'sUser' => $sUser,
			'sPassword' => $sPassword,
			'sDBAuthType' => $sDBAuthType,
			'sDatabaseID' => $sDatabaseID,
			'aDataBases' => $aDataBases,
		];
		$sDriverClassWithNS = "SDExtension\DB\\$sDriverClass";
		return new $sDriverClassWithNS($aConnectionDetails, $sDatabaseID);
		}

	/**
	 * createAccessDsn - Creates MS Access DSN
	 *
	 * @access protected
	 * @param array $aConfiguration Connection Details
	 * @param string $sDatabaseID Database ID
	 * @return string
	 */
	protected static function createAccessDsn($aConfiguration, $sDatabaseID = DB_CATALOG)
		{
		$sMdbFile = "";
		switch ($sDatabaseID)
			{
			case DB_CATALOG:
				$sMdbFile = arr_get($aConfiguration, 'catalogdbfilepath', '');
				break;
			case DB_SHIPPING:
				$sMdbFile = arr_get($aConfiguration, 'shippingdbfilepath', '');
				break;
			}
		return static::createAccessDsnString($sMdbFile);
		}

	/**
	 * createAccessDsnString - Creates MS Access DSN
	 *
	 * @access protected
	 * @param string $sMdbFile MDB File
	 * @return string
	 */
	public static function createAccessDsnString($sMdbFile)
		{
		$sPrefix = static::$m_bPDO ? "odbc:" : "";
		return $sPrefix . "Driver={Microsoft Access Driver (*.mdb)};Dbq=$sMdbFile;Uid=";
		}

	/**
	 * createSQLServerDsn - Creates MS SQL Server DSN
	 *
	 * @access protected
	 * @param array $aConfiguration Connection Details
	 * @param string $sDatabaseID Database ID
	 * @return string
	 */
	protected static function createSQLServerDsn($aConfiguration, $sDatabaseID = DB_CATALOG)
		{
		$sDBName = "";
		switch ($sDatabaseID)
			{
			case DB_CATALOG:
				$sDBName = arr_get($aConfiguration, 'catalogdbsqlname', '');
				break;
			case DB_SHIPPING:
				$sDBName = arr_get($aConfiguration, 'shippingdbsqlname', '');
				break;
			}
		$sServer = arr_get($aConfiguration, 'servername', '');
		return "sqlsrv:server=$sServer;Database=$sDBName";
		}

	}
