<?php

/**
 * ExtensionHelper.php - Contains various global helper functions
 * 
 * @package     SellerDeck Extensions
 * 
 * @author      Péter Erdődi
 * @copyright   © SellerDeck Ltd 2015. All rights reserved.
 */

/**
 * Converts float or double to SellerDeck Number
 *
 * @param   int|float|double    $dNumber    a number
 * @return  int  SellerDeck Number
 */
function float2sdnum($dNumber)
	{
	return (int) round(((double) $dNumber) * 100.00);
	}

/**
 * Converts SellerDeck Number to double
 *
 * @param   int	$nSellerDeckNumber   A SellerDeck Number
 * @return  double   Float number
 */
function sdnum2float($nSellerDeckNumber)
	{
	return (double) ((double) $nSellerDeckNumber) / 100.00;
	}

/**
 * Converts any date to SellerDeck Number
 *
 * @param   int|float|double    $dNumber    a number
 * @return  int  SellerDeck Number
 */
function SellerDeckDate($sDate = null)
	{

	if ($sDate == null)
		{
		return date(SELLERDECK_DATE_FORMAT);
		}
//    echo "$sDate\n";ZZZ
	$sDate = preg_replace('/([0-9]{4}-[0-9]{2}-[0-9]{2}).([0-9]{2}:[0-9]{2}:[0-9]{2}).*/', '$1 $2', $sDate);
//    echo "$sDate\n";
	return date(SELLERDECK_DATE_FORMAT, strtotime($sDate));
	}

/**
 * Gets a column value from the Query Result.
 *
 * @param   PDOStatement   $oQueryResult    Query Result.
 * @param   mixed   $key      The dot-notated key or array of keys
 * @param   string  $default  The default value
 * @return  mixed
 */
function select_get($oQueryResult, $key, $default = null)
	{
	if (!is_object($oQueryResult))
		{
		return $default;
		}
	return arr_get($oQueryResult->fetch(\PDO::FETCH_ASSOC), $key, $default);
	}

/**
 * Gets WSDL file from URL
 *
 * @param   string   $sWsdlUrl    WSDL URL
 * @return  object Wsdl File
 */
function get_wsdl($sWsdlUrl = WSDL_URL)
	{

	$oConnection = curl_init();
	curl_setopt_array($oConnection, array(
		CURLOPT_URL => $sWsdlUrl,
		CURLOPT_BINARYTRANSFER => 1,
		CURLOPT_RETURNTRANSFER => 1,
		CURLOPT_TIMEOUT => 10,
	));
	$sResponsemXml = curl_exec($oConnection);
	if (empty($sResponsemXml))
		{
		throw new \SDExtension\SDException\SDException("Can't get WSDL.");
		}
	return 'data://text/plain;base64,' . base64_encode($sResponsemXml);
//   return $responseXml;
	}

/**
 * Creates a new instance of SDApiSoapJobs
 *
 * @return  CSDApiSoapJobs	New instance of SDApiSoapJobs
 */
function CreateSDApiSoapJobs()
	{
	$sApiKey = \SDExtension\CConfig::get('sdd-configuration.notificationsapi.apikey', '?');
	$sWsdlURL = \SDExtension\CConfig::get('sdd-configuration.notificationsapi.apiwsdl', 'http://localhost:54385/ActinicNotificationService?wsdl');
	$oFileData = get_wsdl($sWsdlURL);
	$oSyncService = new \SoapClient($oFileData, array('exceptions' => false));
	return new \SDExtension\CSDApiSoapJobs($oSyncService, $sApiKey);
	}

/**
 * Saves configuration
 *
 * @param   array   $aSdeConfigData    Configuration Data
 * @return  void
 */
function save_config($aSdeConfigData = [], $bMergeWithOriginal = true, $sSdeConfigFile = EXTENSION_CONFIG_FILE)
	{
	if ($bMergeWithOriginal && file_exists($sSdeConfigFile))
		{
		$aOriginalSdeConfigData = include $sSdeConfigFile;
		$aSdeConfigData = array_merge($aOriginalSdeConfigData, $aSdeConfigData);
		}
	$sConfigPHPCode = sprintf(CONFIG_PHP_HEADER, basename($sSdeConfigFile)) .
			var_export($aSdeConfigData, true) .
			CONFIG_PHP_FOOTER;
	file_put_contents($sSdeConfigFile, $sConfigPHPCode);
	}

/**
 * Gets Table Class Folder by DB Type
 *
 * @return  string	Table Class Folder by DB Type
 */
function GetTableClassFolderByDBType()
	{
	$sDbType = \SDExtension\CConfig::get('sdd-configuration.database.dbtype', DB_TYPE_ACCESS);
	$sTableClassFolder = "access";
	if (DB_TYPE_SQLSERVER == $sDbType)
		{
		$sTableClassFolder = "sqlserver";
		}
	return $sTableClassFolder;
	}

/**
 * SellerDeck Encryption
 *
 * @param  string	Data to encrypt
 * @return  string Encrypted data
 */
function sd_encrypt($sData)
	{
	return encrypt_decrypt('encrypt', $sData);
	}

/**
 * SellerDeck Decryption
 *
 * @param  string	Data to decrypt
 * @return  string Decrypted data
 */
function sd_decrypt($sData)
	{
	return encrypt_decrypt('decrypt', $sData);
	}

/**
 * Get Setting
 *
 * @param	string   $sSettingKey Setting key
 * @param	mixed	   $vDefault Default setting value
 * @return  mixed	   Setting value
 */
function get_setting($sSettingKey, $vDefault = null)
	{
	$aSettings = parse_ini_file(SETTINGS_INI);
	return arr_get($aSettings, $sSettingKey, $vDefault);
	}

/**
 * Set Setting
 *
 * @param	string   $sSettingKey Setting key
 * @param	mixed	   $vValue Setting value
 * @return  void
 */
function set_setting($sSettingKey, $vValue)
	{
	$aSettings = parse_ini_file(SETTINGS_INI);
	$aSettings[$sSettingKey] = $vValue;
	write_ini_file(['extension' => $aSettings], SETTINGS_INI, true);
	}

/**
 * Is PDO?
 *
 * @return  bool PDO
 */
function is_PDO()
	{
//
// We always use PDO
// 
// return in_array(strtolower(get_setting('pdo', '1')), ['1', 'true', 'on', 'yes']);
//
	return true;
	}

/**
 * encrypt_decrypt - Encrypts or decrypts data
 *
 * @param string $sAction Action (encrypt/decrypt)
 * @param string $sData Data
 * @return  string  Encrypted or decrypted data
 */
function encrypt_decrypt($sAction, $sData)
	{
	$sOutput = "";
	$sEncryptMethod = "AES-256-CBC";
	$sSecretKey = 'This is my secret key';
	$sSecretIV = 'This is my secret iv';
//
//  hash
//
	$sKey = hash('sha256', $sSecretKey);
//
//  iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
//
	$sIV = substr(hash('sha256', $sSecretIV), 0, 16);
	if ($sAction == 'encrypt')
		{
		$sOutput = base64_encode(openssl_encrypt($sData, $sEncryptMethod, $sKey, 0, $sIV));
		}
	else if ($sAction == 'decrypt')
		{
		$sOutput = openssl_decrypt(base64_decode($sData), $sEncryptMethod, $sKey, 0, $sIV);
		}
	return $sOutput;
	}

/**
 * valid_date - Check date validity
 *
 * @param string $sDate Date
 * @return  bool Valid
 */
function valid_date($sDate)
	{
	list($nY, $sM, $sD) = explode('-', $sDate);
	return checkdate($sM, $sD, $nY);
	}

/**
 * valid_date_format - Check date format validity
 *
 * @param string $sDate Date
 * @return  bool Valid format
 */
function valid_date_format($sDate)
	{
	if (preg_match("/^([0-9]{4})-(0?[1-9]|1[0-2])-(0?[1-9]|[1-2][0-9]|3[0-1])$/", $sDate))
		{
		return true;
		}
	return false;
	}

/**
 * between - Check if values are in an interval
 *
 * @param mixed $vInput Input
 * @param numeric $vParam1 Parameter 1
 * @param numeric $vParam2 Parameter 2
 * @return  bool Between params
 */
function between($vInput, $vParam1, $vParam2)
	{
	if (($vInput >= $vParam1) && ($vInput <= $vParam2))
		{
		return true;
		}
	return false;
	}

/**
 * CreateLock - Creates lock file
 *
 * @param string $sDownloadLockFile Download Lock File
 * @return  bool Success
 */
function CreateLock($sDownloadLockFile = null)
	{
	if ($sDownloadLockFile == null)
		{
		$sDownloadLockFile = get_setting('extension_program_data', "./") . LOCK_FILE;
		}
	if (!file_exists($sDownloadLockFile) || ((time() - filemtime($sDownloadLockFile)) > LOCK_FILE_MAX_AGE))
		{
		file_put_contents($sDownloadLockFile, "");
		return true;
		}
	return false;
	}

/**
 * RemoveLock - Removes lock file
 *
 * @param string $sDownloadLockFile Download Lock File
 * @return  bool Success
 */
function RemoveLock($sDownloadLockFile = null)
	{
	if ($sDownloadLockFile == null)
		{
		$sDownloadLockFile = get_setting('extension_program_data', "./") . LOCK_FILE;
		}
	if (file_exists($sDownloadLockFile))
		{
		unlink($sDownloadLockFile);
		return true;
		}
	return false;
	}
